@@ -46,7 +46,7 @@ def prody_anm(pdb, **kwargs):
         if key not in kwargs:
             kwargs[key] = DEFAULTS[key]
 
-    from os.path import isdir, join
+    from os.path import isdir, join, exists
     outdir = kwargs.get('outdir')
     if not isdir(outdir):
         raise IOError('{0} is not a valid path'.format(repr(outdir)))
@@ -64,6 +64,10 @@ def prody_anm(pdb, **kwargs):
     altloc = kwargs.get('altloc')
     zeros = kwargs.get('zeros')
     turbo = kwargs.get('turbo')
+    membrane = kwargs.get('membrane')
+
+    if membrane and not exists(pdb):
+        pdb = prody.fetchPDBfromOPM(pdb)
 
     pdb = prody.parsePDB(pdb, model=model, altloc=altloc)
     if prefix == '_anm':
@@ -77,6 +81,10 @@ def prody_anm(pdb, **kwargs):
     LOGGER.info('{0} atoms will be used for ANM calculations.'
                 .format(len(select)))
 
+    if membrane:
+        anm = prody.exANM(pdb.getTitle())
+    else:
+        anm = prody.ANM(pdb.getTitle())
     try:
         gamma = float(kwargs.get('gamma'))
         LOGGER.info("Using gamma {0}".format(gamma))
@@ -103,8 +111,6 @@ def prody_anm(pdb, **kwargs):
         except TypeError:
             raise TypeError("Please provide cutoff as a float or equation using math")
 
-    anm = prody.ANM(pdb.getTitle())
-
     nproc = kwargs.get('nproc')
     anm.buildHessian(select, cutoff, gamma, sparse=sparse, kdtree=kdtree)
     anm.calcModes(nmodes, zeros=zeros, turbo=turbo, nproc=nproc)
@@ -252,7 +258,7 @@ for key in _:
 def addCommand(commands):
 
     subparser = commands.add_parser('anm',
-        help='perform anisotropic network model calculations')
+        help='perform anisotropic network model normal mode analysis calculations')
 
     subparser.add_argument('--quiet', help="suppress info messages to stderr",
         action=Quiet, nargs=0)
