@@ -346,7 +346,10 @@ def extendAtoms(nodes, atoms, is3d=False):
         if res is None:
             raise ValueError('atoms must contain a residue for all atoms')
         if isinstance(res, list):
-            raise ValueError('not enough data to get a single residue for all atoms')
+            if len(res) == 1:
+                res = res[0]
+            else:
+                raise ValueError('not enough data to get a single residue for all atoms')
 
         res_atom_indices = res._getIndices()
         if not fastin(res, residues):
