@@ -716,7 +716,7 @@ for func in _[1:]:
     FUNCNAMES_OPLIST = FUNCNAMES_OPLIST | kwfunc
     FUNCNAMES_EXPR += ~kwfunc
 
-RE_SCHARS = re_compile('`[\w\W]*?`')
+RE_SCHARS = re_compile(r'`[\w\W]*?`')
 PP_SCHARS = pp.Regex(RE_SCHARS)
 
 
@@ -735,7 +735,7 @@ def specialCharsParseAction(sel, loc, token):
 PP_SCHARS.setParseAction(specialCharsParseAction)
 
 
-RE_REGEXP = re_compile('"[\w\W]*"')
+RE_REGEXP = re_compile(r'"[\w\W]*"')
 PP_REGEXP = pp.Regex(RE_REGEXP.pattern)
 
 
@@ -754,11 +754,11 @@ def regularExpParseAction(sel, loc, token):
 
 PP_REGEXP.setParseAction(regularExpParseAction)
 
-_ = '[-+]?\d+(\.\d*)?([eE]\d+)?'
+_ = r'[-+]?\d+(\.\d*)?([eE]\d+)?'
 RE_NRANGE = re_compile(_ + '\ *(to|:)\ *' + _)
 
 PP_NRANGE = pp.Group(pp.Regex(RE_NRANGE.pattern) +
-                     pp.Optional(pp.Regex('(\ *:\ *' + _ + ')')))
+                     pp.Optional(pp.Regex(r'(\ *:\ *' + _ + ')')))
 
 
 def rangeParseAction(sel, loc, tokens):
@@ -1341,6 +1341,7 @@ class Select(object):
         isDataLabel = atoms.isDataLabel
         append = None
         wasand = False
+        wasdata = False
         while tokens:
             # check whether token is an array to avoid array == str comparison
             token = tokens.pop(0)
@@ -1354,9 +1355,10 @@ class Select(object):
                             .format(repr('and ... and')), ['and', 'and'])
                     append = None
                     wasand = True
+                    wasdata = False
                     continue
 
-                elif isFlagLabel(token):
+                elif isFlagLabel(token) and not wasdata:
                     flags.append(token)
                     append = None
 
@@ -1396,10 +1398,12 @@ class Select(object):
                         evals.append([])
                         append = evals[-1].append
                     append(token)
+                    wasdata = True
 
                 elif token in UNARY:
                     unary.append([])
                     append = unary[-1].append
+                    wasdata = False
 
                     if token == 'not':
                         append((token,))
@@ -2420,3 +2424,10 @@ class Select(object):
         if self._coords is None:
             self._coords = self._atoms._getCoords()
         return self._coords
+
+    def _getAnisous(self):
+        """Returns anisotropic temperature factors of atoms."""
+
+        if self._anisous is None:
+            self._anisous = self._atoms._getAnisous()
+        return self._anisous
