@@ -47,7 +47,7 @@ The following classes are designed for modeling and analysis of protein dynamics
 
 Usage of these classes are shown in :ref:`exanm` and :ref:`imanm` examples.
 
-SignDy
+Signature Dynamics (SignDy)
 ===============
 
 The following classes are designed for signature dynamics analysis of protein/domain families, 
@@ -119,6 +119,14 @@ The following class and its functions can be used to generate conformers using a
 
   * :class:`.AdaptiveANM` - generate transitions between two conformers using best overlapping modes 
 
+ENM-MD hybrid methods
+===================
+
+The following classes and functions can be used to generate conformers using ENM-MD hybrid methods:
+
+  * :class:`.ClustENM` - generate conformers by exploring combinations of modes and clustering, using minimisation and optionally MD
+  * :function:`.ANMD` - generate conformers by traversing a number of individual modes, applying minimisation to each conformer
+
 Essential Site Scanning Analysis (ESSA)
 ========================================
 
@@ -360,3 +368,14 @@ __all__.extend(essa.__all__)
 from prody.ensemble import functions
 functions.ClustENM = ClustENM
 
+from . import lda
+from .lda import *
+__all__.extend(lda.__all__)
+
+from . import logistic
+from .logistic import *
+__all__.extend(logistic.__all__)
+
+from . import anmd
+from .anmd import *
+__all__.extend(anmd.__all__)
\ No newline at end of file
