@@ -19,7 +19,8 @@ from .mode import VectorBase, Mode, Vector
 from .gnm import GNMBase
 
 __all__ = ['calcCollectivity', 'calcCovariance', 'calcCrossCorr',
-           'calcFractVariance', 'calcSqFlucts', 'calcRMSFlucts', 'calcTempFactors',
+           'calcFractVariance', 'calcSqFlucts', 'calcRMSFlucts',
+           'calcMostMobileNodes', 'calcTempFactors',
            'calcProjection', 'calcCrossProjection',
            'calcSpecDimension', 'calcPairDeformationDist',
            'calcDistFlucts', 'calcHinges', 'calcHitTime',
@@ -343,6 +344,28 @@ def calcRMSFlucts(modes):
 
     return sq_flucts ** 0.5
 
+def calcMostMobileNodes(modes, **kwargs):
+    """Returns indices for nodes with highest root mean square fluctuations (RMSFs) for given set of normal *modes*
+    above a particular *percentile* and/or *cutoff*.
+
+    :arg percentile: percentile for internal cutoff (between 0 and 100).
+        Default 0 takes all values
+    :type percentile: int
+
+    :arg cutoff: user-defined cutoff, default is to take all values
+    :type cutoff: float
+    """
+    rmsf = calcRMSFlucts(modes)
+
+    cutoff = kwargs.get('cutoff', rmsf.min())
+    inds = np.nonzero(rmsf > cutoff)[0]
+
+    percentile = kwargs.get('percentile', 0)
+    cutoff = np.percentile(rmsf, percentile)
+    inds = inds[np.nonzero(rmsf[inds] > cutoff)[0]]
+
+    return inds
+
 def calcCrossCorr(modes, n_cpu=1, norm=True):
     """Returns cross-correlations matrix.  For a 3-d model, cross-correlations
     matrix is an NxN matrix, where N is the number of atoms.  Each element of
@@ -407,12 +430,12 @@ def calcCrossCorr(modes, n_cpu=1, norm=True):
             import multiprocessing
             n_cpu = min(multiprocessing.cpu_count(), n_cpu)
             queue = multiprocessing.Queue()
-            size = n_modes / n_cpu
+            size = n_modes // n_cpu
             for i in range(n_cpu):
                 if n_cpu - i == 1:
-                    indices = modes.indices[i*size:]
+                    indices = modes.getIndices()[i*size:]
                 else:
-                    indices = modes.indices[i*size:(i+1)*size]
+                    indices = modes.getIndices()[i*size:(i+1)*size]
                 process = multiprocessing.Process(
                     target=_crossCorrelations,
                     args=(queue, n_atoms, array, variances, indices))
