@@ -180,7 +180,7 @@ def trimModelByMask(model, mask):
 
     if not isListLike(mask):
         raise TypeError('mask must be either a list or a numpy.ndarray, not {0}'
-                        .format(type(model)))
+                        .format(type(mask)))
     
     is_bool = mask.dtype is np.dtype('bool')
 
@@ -279,7 +279,7 @@ def sliceMode(mode, atoms, select):
 
 def sliceModel(model, atoms, select, norm=False):
     """Returns a part of the *model* (modes calculated) for *atoms* matching *select*. 
-    Note that normal modes are sliced instead the connectivity matrix. Sliced normal 
+    Note that normal modes are sliced instead of the connectivity matrix. Sliced normal 
     modes (eigenvectors) are not normalized unless *norm* is **True**.
 
     :arg mode: NMA model instance to be sliced
@@ -328,7 +328,7 @@ def sliceModelByMask(model, mask, norm=False):
 
     if not isListLike(mask):
         raise TypeError('mask must be either a list or a numpy.ndarray, not {0}'
-                        .format(type(model)))
+                        .format(type(mask)))
     
     is_bool = mask.dtype is np.dtype('bool')
 
@@ -425,7 +425,7 @@ def reduceModelByMask(model, mask):
     
     if not isListLike(mask):
         raise TypeError('mask must be either a list or a numpy.ndarray, not {0}'
-                        .format(type(model)))
+                        .format(type(mask)))
     
     is_bool = mask.dtype is np.dtype('bool')
 
