@@ -9,12 +9,18 @@ import numpy as np
 from prody.proteins import alignChains
 from prody.utilities import openFile, showFigure, copy, isListLike, pystr, DTYPE
 from prody import LOGGER, SETTINGS
-from prody.atomic import Atomic, AtomMap, Chain, AtomGroup, Selection, Segment, Select, AtomSubset
+from prody.atomic import Atomic, AtomGroup
 from prody.sequence import buildSeqidMatrix
 
-from .ensemble import *
-from .pdbensemble import *
-from .conformation import *
+have_openmm = True
+try:
+    import openmm
+except ImportError:
+    have_openmm = False
+
+from .ensemble import Ensemble
+from .pdbensemble import PDBEnsemble
+from .conformation import PDBConformation
 
 __all__ = ['saveEnsemble', 'loadEnsemble', 'trimPDBEnsemble',
            'calcOccupancies', 'showOccupancies',
@@ -46,7 +52,9 @@ def saveEnsemble(ensemble, filename=None, **kwargs):
                           '_padding', '_ionicStrength', '_force_field', '_tolerance',
                           '_maxIterations', '_sim', '_temp', '_t_steps', '_outlier',
                           '_mzscore', '_v1', '_parallel', '_idx_cg', '_n_cg', '_cycle',
-                          '_time', '_targeted', '_tmdk'])
+                          '_time', '_targeted', '_tmdk', '_cc'])
+        if have_openmm:
+            attr_list.extend(['_topology', '_positions'])
 
     if filename is None:
         filename = ensemble.getTitle().replace(' ', '_')
@@ -58,6 +66,18 @@ def saveEnsemble(ensemble, filename=None, **kwargs):
 
     atoms = dict_['_atoms']
     if atoms is not None:
+
+        if isinstance(atoms, AtomGroup):
+            ag = atoms
+        else:
+            ag = atoms._ag
+
+        if not hasattr(atoms, '_bondOrders'):
+            ag._bondOrders = None
+
+        if not hasattr(atoms, '_bondIndex'):
+            ag._bondIndex = None
+
         attr_dict['_atoms'] = np.array([atoms, None], 
                                         dtype=object)
 
@@ -158,7 +178,10 @@ def loadEnsemble(filename, **kwargs):
                     '_rmsd', '_n_gens', '_maxclust', '_threshold', '_sol',
                     '_sim', '_temp', '_t_steps', '_outlier', '_mzscore', '_v1',
                     '_parallel', '_idx_ca', '_n_ca', '_cycle', '_time', '_targeted',
-                    '_tmdk']
+                    '_tmdk', '_cc']
+        if have_openmm:
+            attrs.extend(['_topology', '_position'])
+            
             
             for attr in attrs:
                 if attr in attr_dict.files:
@@ -172,10 +195,18 @@ def loadEnsemble(filename, **kwargs):
         atoms = attr_dict['_atoms'][0]
 
         if isinstance(atoms, AtomGroup):
-            data = atoms._data
+            ag = atoms
         else:
-            data = atoms._ag._data
-        
+            ag = atoms._ag
+
+        if not hasattr(ag, '_bondOrders'):
+            ag._bondOrders = None
+
+        if not hasattr(ag, '_bondIndex'):
+            ag._bondIndex = None
+
+        data = ag._data
+
         for key in data:
             arr = data[key]
             char = arr.dtype.char
@@ -400,14 +431,14 @@ def buildPDBEnsemble(atomics, ref=None, title='Unknown', labels=None, atommaps=N
 
     if 'mapping_func' in kwargs:
         raise DeprecationWarning('mapping_func is deprecated. Please see release notes for '
-                                 'more details: http://prody.csb.pitt.edu/manual/release/v1.11_series.html')
+                                 'more details: http://www.bahargroup.org/prody/manual/release/v1.11_series.html')
     start = time.time()
 
     if not isListLike(atomics):
         raise TypeError('atomics should be list-like')
 
     if len(atomics) == 1 and degeneracy is True:
-        raise ValueError('atomics should have at least two items')
+        raise ValueError('atomics should have at least two items or degeneracy should be False')
 
     if labels is not None:
         if len(labels) != len(atomics):
