@@ -101,6 +101,36 @@ Use the following to parse and access header data in PDB files:
   * :class:`.Polymer` - store PDB polymer (macromolecule) component data
   * :class:`.DBRef` - store polymer sequence database reference records
 
+Analyze interactions and stability with InSty and find water bridges with WatFinder
+====================
+
+Use the following to analyze interactions within protein structure or
+between protein and ligand structure in single PDB file or in trajectory:
+
+  * :func:`.addHydrogens` - add missing hydrogens to :file:`.pdb` files
+  * :func:`.calcHydrogenBonds` - compute hydrogen bonds in proteins
+  * :func:`.calcSaltBridges` - compute salt bridges in proteins
+  * :func:`.calcRepulsiveIonicBonding` - compute repulsive ionic bonding in proteins
+  * :func:`.calcPiStacking` - compute Pi-stacking interactions in proteins
+  * :func:`.calcPiCation` - compute Pi-cation interactions in proteins
+  * :func:`.calcHydrophobic` - compute hydrophobic interactions in proteins
+  * :func:`.calcProteinInteractions` - compute all above interaction types at once
+  * :func:`.showProteinInteractions_VMD` - return TCL file for visualization in VMD 
+
+  * :func:`.calcHydrogenBondsDCD` - compute hydrogen bonds in a trajectory for proteins
+  * :func:`.calcSaltBridgesDCD` - ompute salt bridges in a trajectory for proteins
+  * :func:`.calcRepulsiveIonicBondingDCD` - compute repulsive ionic bonding in a trajectory for proteins    
+  * :func:`.calcPiStackingDCD` - compute Pi-stacking interactions in a trajectory for proteins
+  * :func:`.calcPiCationDCD` - compute Pi-cation interactions in a trajectory for proteins
+  * :func:`.calcHydrophobicDCD` - compute hydrophobic interactions in a trajectory for proteins
+  * :func:`.calcStatisticsInteractions` - return statistical information for each interaction type 
+
+  * :func:`.calcLigandInteractions` - compute all type of interactions between protein and ligand
+  * :func:`.listLigandInteractions` - return list of interactions between protein and ligand
+  * :func:`.showLigandInteraction_VMD` - return TCL file for visualization of interactions for VMD
+
+  * :class:`.Interactions` - store inteactions for a single PDB structure
+  * :class:`.InteractionsDCD` - store interactions for a trajectory
 
 Compare/align chains
 ====================
@@ -204,6 +234,10 @@ from . import pdbfile
 from .pdbfile import *
 __all__.extend(pdbfile.__all__)
 
+from . import mmtffile
+from .mmtffile import *
+__all__.extend(mmtffile.__all__)
+
 from . import emdfile
 from .emdfile import *
 __all__.extend(emdfile.__all__)
@@ -212,17 +246,25 @@ from . import ciffile
 from .ciffile import *
 __all__.extend(ciffile.__all__)
 
+_getPDBid  = ciffile._getPDBid = pdbfile._getPDBid
+long_id_check_str = ciffile.long_id_check_str = pdbfile.long_id_check_str
+wwpdb.long_id_check_str = long_id_check_str
+
 from . import starfile
 from .starfile import *
 __all__.extend(starfile.__all__)
 
+from . import interactions
+from .interactions import *
+__all__.extend(interactions.__all__)
+
 try:
     from . import waterbridges
     from .waterbridges import *
 except SyntaxError:
     import logging
     logger = logging.getLogger()
-    logger.warn("Cannot import waterbridges in python 2")
+    logger.warn("Cannot import waterbridges")
 else:
     __all__.extend(waterbridges.__all__)
 
@@ -232,3 +274,7 @@ __all__.extend(fixer.__all__)
 
 from .pdbfile import PDBParseError
 from .ciffile import MMCIFParseError
+
+from . import opm
+from .opm import *
+__all__.extend(opm.__all__)
