@@ -15,11 +15,11 @@ from prody.utilities import openFile
 from prody import LOGGER, SETTINGS
 
 from .localpdb import fetchPDB
-from .starfile import parseSTARLines, StarDict, parseSTARSection
+from .starfile import parseSTARSection
 from .cifheader import getCIFHeaderDict
-from .header import buildBiomolecules, assignSecstr, isHelix, isSheet
+from .header import buildBiomolecules, assignSecstr
 
-__all__ = ['parseMMCIFStream', 'parseMMCIF']
+__all__ = ['parseMMCIFStream', 'parseMMCIF', 'parseCIF', 'writeMMCIF']
 
 
 class MMCIFParseError(Exception):
@@ -36,6 +36,11 @@ _parseMMCIFdoc = """
         chains are parsed
     :type chain: str
 
+    :arg segment: segment identifiers for parsing specific chains, e.g.
+        ``segment='A'``, ``segment='B'``, ``segment='DE'``, by default all
+        segment are parsed
+    :type segment: str
+
     :arg subset: a predefined keyword to parse subset of atoms, valid keywords
         are ``'calpha'`` (``'ca'``), ``'backbone'`` (``'bb'``), or **None**
         (read all atoms), e.g. ``subset='bb'``
@@ -46,10 +51,17 @@ _parseMMCIFdoc = """
 
     :arg altloc: if a location indicator is passed, such as ``'A'`` or ``'B'``,
          only indicated alternate locations will be parsed as the single
-         coordinate set of the AtomGroup, if *altloc* is set **True** all
+         coordinate set of the AtomGroup, if *altloc* is set ``'all'`` then all
          alternate locations will be parsed and each will be appended as a
          distinct coordinate set, default is ``"A"``
     :type altloc: str
+
+    :arg unite_chains: unite chains with the same segment name (auth_asym_id), making chain ids be 
+        auth_asym_id instead of label_asym_id. This can be helpful in some cases e.g. alignments, but can 
+        cause some problems too. For example, using :meth:`.buildBiomolecules` afterwards requires original 
+        chain id (label_asym_id). Using biomol=True, inside parseMMCIF is fine.
+        Default is *False*
+    :type unite_chains: bool
     """
 
 _PDBSubsets = {'ca': 'ca', 'calpha': 'ca', 'bb': 'bb', 'backbone': 'bb'}
@@ -66,41 +78,36 @@ def parseMMCIF(pdb, **kwargs):
     :arg pdb: a PDB identifier or a filename
         If needed, mmCIF files are downloaded using :func:`.fetchPDB()` function.
     :type pdb: str
-
-    :arg chain: comma separated string or list-like of chain IDs
-    :type chain: str, tuple, list, :class:`~numpy.ndarray`
-
-    :arg unite_chains: unite chains with the same segment name
-        Default is *False*
-    :type unite_chains: bool
     """
     chain = kwargs.pop('chain', None)
+    segment = kwargs.pop('segment', None)
     title = kwargs.get('title', None)
     unite_chains = kwargs.get('unite_chains', False)
     auto_bonds = SETTINGS.get('auto_bonds')
     get_bonds = kwargs.get('bonds', auto_bonds)
     if get_bonds:
-        LOGGER.warn('Parsing struct_conn information from mmCIF is current unsupported and no bond information is added to the results')
+        LOGGER.warn('Parsing struct_conn information from mmCIF is currently unsupported and no bond information is added to the results')
     if not os.path.isfile(pdb):
-        if len(pdb) == 5 and pdb.isalnum():
+        if (len(pdb) == 5 and pdb.isalnum()) or eval(long_id_check_str % 13):
             if chain is None:
-                chain = pdb[-1]
-                pdb = pdb[:4]
+                pdb, chain = _getPDBid(pdb)
             else:
                 raise ValueError('Please provide chain as a keyword argument '
                                  'or part of the PDB ID, not both')
-        else:
-            chain = chain
 
-        if len(pdb) == 4 and pdb.isalnum():
+        if (len(pdb) == 4 and pdb.isalnum()) or eval(long_id_check_str % 12):
             if title is None:
                 title = pdb
                 kwargs['title'] = title
 
             if os.path.isfile(pdb + '.cif'):
                 filename = pdb + '.cif'
+                LOGGER.debug('CIF file is found in working directory ({0}).'
+                            .format(filename))
             elif os.path.isfile(pdb + '.cif.gz'):
                 filename = pdb + '.cif.gz'
+                LOGGER.debug('CIF file is found in working directory ({0}).'
+                            .format(filename))
             else:
                 filename = fetchPDB(pdb, report=True,
                                     format='cif', compressed=False)
@@ -119,12 +126,30 @@ def parseMMCIF(pdb, **kwargs):
             title = title[3:]
         kwargs['title'] = title
     cif = openFile(pdb, 'rt')
-    result = parseMMCIFStream(cif, chain=chain, **kwargs)
+    result = parseMMCIFStream(cif, chain=chain, segment=segment, **kwargs)
     cif.close()
     if unite_chains:
-        result.setSegnames(result.getChids())
+        if isinstance(result, AtomGroup):
+            result.setChids(result.getSegnames())
+
+        elif isinstance(result, list):
+            # e.g. multiple biomol assemblies
+            [r.setChids(r.getSegnames()) for r in result if isinstance(r, AtomGroup)]
+
+        elif isinstance(result, tuple):
+            # atoms, header
+            if isinstance(result[0], AtomGroup):
+                result[0].setChids(result[0].getSegnames())
+
+            elif isinstance(result[0], list):
+                # e.g. multiple biomol assemblies
+                [r.setChids(r.getSegnames()) for r in result[0] if isinstance(r, AtomGroup)]
+
+        elif result is not None:
+            raise TypeError('result from parseMMCIFStream should be a tuple, AtomGroup or list')
     return result
 
+parseCIF = parseMMCIF
 
 def parseMMCIFStream(stream, **kwargs):
     """Returns an :class:`.AtomGroup` and/or a class:`.StarDict` 
@@ -138,8 +163,11 @@ def parseMMCIFStream(stream, **kwargs):
     model = kwargs.get('model')
     subset = kwargs.get('subset')
     chain = kwargs.get('chain')
+    segment = kwargs.get('segment')
+    unite_chains = kwargs.get('unite_chains', False)
     altloc = kwargs.get('altloc', 'A')
     header = kwargs.get('header', False)
+    report = kwargs.get('report', False)
     assert isinstance(header, bool), 'header must be a boolean'
 
     if model is not None:
@@ -150,6 +178,15 @@ def parseMMCIFStream(stream, **kwargs):
             raise TypeError('model must be an integer, {0} is invalid'
                             .format(str(model)))
     title_suffix = ''
+
+
+    if chain is not None:
+        if not isinstance(chain, str):
+            raise TypeError('chain must be a string')
+        elif len(chain) == 0:
+            raise ValueError('chain must not be an empty string')
+        title_suffix = chain + title_suffix
+
     if subset:
         try:
             subset = _PDBSubsets[subset.lower()]
@@ -159,12 +196,13 @@ def parseMMCIFStream(stream, **kwargs):
             raise ValueError('{0} is not a valid subset'
                              .format(repr(subset)))
         title_suffix = '_' + subset
-    if chain is not None:
-        if not isinstance(chain, str):
-            raise TypeError('chain must be a string')
-        elif len(chain) == 0:
-            raise ValueError('chain must not be an empty string')
-        title_suffix = '_' + chain + title_suffix
+
+    if segment is not None:
+        if not isinstance(segment, str):
+            raise TypeError('segment must be a string')
+        elif len(segment) == 0:
+            raise ValueError('segment must not be an empty string')
+        title_suffix = '_' + segment + title_suffix
 
     ag = None
     if 'ag' in kwargs:
@@ -194,7 +232,8 @@ def parseMMCIFStream(stream, **kwargs):
         if header or biomol or secondary:
             hd = getCIFHeaderDict(lines)
 
-        _parseMMCIFLines(ag, lines, model, chain, subset, altloc)
+        _parseMMCIFLines(ag, lines, model, chain, subset, altloc, 
+                         segment, unite_chains, report)
 
         if ag.numAtoms() > 0:
             LOGGER.report('{0} atoms and {1} coordinate set(s) were '
@@ -236,13 +275,19 @@ def parseMMCIFStream(stream, **kwargs):
 
 
 parseMMCIFStream.__doc__ += _parseMMCIFdoc
+parseMMCIF.__doc__ += _parseMMCIFdoc
 
 
 def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
-                     altloc_torf):
+                     altloc_torf, segment, unite_chains,
+                     report):
     """Returns an AtomGroup. See also :func:`.parsePDBStream()`.
 
     :arg lines: mmCIF lines
+
+    :arg report: whether to report warnings about not finding data
+        default False
+    :type report: bool
     """
 
     if subset is not None:
@@ -265,15 +310,17 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
     stop = 0
     while not doneAtomBlock:
         line = lines[i]
-        if line[:11] == '_atom_site.':
+        if line.strip()[:11] == '_atom_site.':
             fieldCounter += 1
             fields[line.split('.')[1].strip()] = fieldCounter
             foundAtomFields = True
 
-        elif foundAtomFields and line.strip() != '#':
+        elif foundAtomFields and (line.startswith("HETATM") or line.startswith("ATOM")):
             if not foundAtomBlock:
                 foundAtomBlock = True
                 start = i
+            if i + 1 < len(lines) and not lines[i + 1].startswith(("ATOM", "HETATM", "#", "_")):
+                line = line.strip() + " " + lines[i + 1].strip() + "\n"
             models.append(line.split()[fields['pdbx_PDB_model_num']])
             if len(models) == 1 or (models[asize] != models[asize-1]):
                 nModels += 1
@@ -319,12 +366,12 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
         elif altloc_torf.strip() != 'A':
             LOGGER.info('Parsing alternate locations {0}.'
                         .format(altloc_torf))
-            which_altlocs = '.' + ''.join(altloc_torf.split())
+            which_altlocs = ' ' + ''.join(altloc_torf.split())
         else:
-            which_altlocs = '.A'
+            which_altlocs = ' A'
         altloc_torf = False
     else:
-        which_altlocs = '.A'
+        which_altlocs = ' A'
         altloc_torf = True
 
     coordinates = np.zeros((asize, 3), dtype=float)
@@ -375,18 +422,25 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
             if not (atomname in subset and resname in protein_resnames):
                 continue
 
-        chID = line.split()[fields['auth_asym_id']]
+        chID = line.split()[fields['label_asym_id']]
+        segID = line.split()[fields['auth_asym_id']]
+
         if chain is not None:
             if isinstance(chain, str):
                 chain = chain.split(',')
             if not chID in chain:
+                if not unite_chains:
+                    continue
+                if not segID in chain:
+                    continue
+
+        if segment is not None:
+            if isinstance(segment, str):
+                segment = segment.split(',')
+            if not segID in segment:
                 continue
 
-        segID = line.split()[fields['label_asym_id']]
-
         alt = line.split()[fields['label_alt_id']]
-        if alt not in which_altlocs and which_altlocs != 'all':
-            continue
 
         if alt == '.':
             alt = ' '
@@ -424,8 +478,10 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
 
         serials[acount] = line.split()[fields['id']]
         elements[acount] = line.split()[fields['type_symbol']]
-        bfactors[acount] = line.split()[fields['B_iso_or_equiv']]
-        occupancies[acount] = line.split()[fields['occupancy']]
+        if 'B_iso_or_equiv' in fields.keys():
+            bfactors[acount] = line.split()[fields['B_iso_or_equiv']]
+        if 'occupancy' in fields.keys():
+            occupancies[acount] = line.split()[fields['occupancy']]
 
         acount += 1
 
@@ -434,47 +490,52 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
     else:
         modelSize = acount
 
+    mask = np.full(acount, True, dtype=bool)
+    if which_altlocs != 'all':
+        #mask out any unwanted alternative locations
+        mask = (altlocs == ' ') | np.logical_or(*[(altlocs == altloc)
+                                                  for altloc in which_altlocs])
+
+    if np.all(mask == False):
+        mask = (altlocs == altlocs[0])
+
     if addcoords:
-        atomgroup.addCoordset(coordinates[:modelSize])
+        atomgroup.addCoordset(coordinates[mask][:modelSize])
     else:
-        atomgroup._setCoords(coordinates[:modelSize])
+        atomgroup._setCoords(coordinates[mask][:modelSize])
 
-    atomgroup.setNames(atomnames[:modelSize])
-    atomgroup.setResnames(resnames[:modelSize])
-    atomgroup.setResnums(resnums[:modelSize])
-    atomgroup.setSegnames(segnames[:modelSize])
-    atomgroup.setChids(chainids[:modelSize])
-    atomgroup.setFlags('hetatm', hetero[:modelSize])
-    atomgroup.setFlags('pdbter', termini[:modelSize])
-    atomgroup.setFlags('selpdbter', termini[:modelSize])
-    atomgroup.setAltlocs(altlocs[:modelSize])
-    atomgroup.setIcodes(icodes[:modelSize])
-    atomgroup.setSerials(serials[:modelSize])
+    atomgroup.setNames(atomnames[mask][:modelSize])
+    atomgroup.setResnames(resnames[mask][:modelSize])
+    atomgroup.setResnums(resnums[mask][:modelSize])
+    atomgroup.setSegnames(segnames[mask][:modelSize])
+    atomgroup.setChids(chainids[mask][:modelSize])
+    atomgroup.setFlags('hetatm', hetero[mask][:modelSize])
+    atomgroup.setFlags('pdbter', termini[mask][:modelSize])
+    atomgroup.setFlags('selpdbter', termini[mask][:modelSize])
+    atomgroup.setAltlocs(altlocs[mask][:modelSize])
+    atomgroup.setIcodes(icodes[mask][:modelSize])
+    atomgroup.setSerials(serials[mask][:modelSize])
 
-    atomgroup.setElements(elements[:modelSize])
+    atomgroup.setElements(elements[mask][:modelSize])
     from prody.utilities.misctools import getMasses
-    atomgroup.setMasses(getMasses(elements[:modelSize]))
-    atomgroup.setBetas(bfactors[:modelSize])
-    atomgroup.setOccupancies(occupancies[:modelSize])
+    atomgroup.setMasses(getMasses(elements[mask][:modelSize]))
+    atomgroup.setBetas(bfactors[mask][:modelSize])
+    atomgroup.setOccupancies(occupancies[mask][:modelSize])
 
     anisou = None
     siguij = None
-    try:
-        data = parseSTARSection(lines, "_atom_site_anisotrop")
-        x = data[0] # check if data has anything in it
-    except IndexError:
-        LOGGER.warn("No anisotropic B factors found")
-    else:
-        anisou = np.zeros((acount, 6),
-                          dtype=ATOMIC_FIELDS['anisou'].dtype)
+    data = parseSTARSection(lines, "_atom_site_anisotrop", report=report)
+    if len(data) > 0:
+        anisou = np.zeros((asize, 6),
+                           dtype=float)
         
         if "_atom_site_anisotrop.U[1][1]_esd" in data[0].keys():
-            siguij = np.zeros((acount, 6),
+            siguij = np.zeros((asize, 6),
                               dtype=ATOMIC_FIELDS['siguij'].dtype)
 
         for entry in data:
             try:
-                index = np.where(atomgroup.getSerials() == int(
+                index = np.where(serials == int(
                     entry["_atom_site_anisotrop.id"]))[0][0]
             except:
                 continue
@@ -497,13 +558,47 @@ def _parseMMCIFLines(atomgroup, lines, model, chain, subset,
                 except:
                     pass
 
-        atomgroup.setAnisous(anisou) # no division needed anymore
+        atomgroup.setAnisous(anisou[mask][:modelSize]) # no division needed anymore
 
-        if not np.any(siguij):
-            atomgroup.setAnistds(siguij)  # no division needed anymore
+        if np.any(siguij):
+            atomgroup.setAnistds(siguij[mask][:modelSize])  # no division needed anymore
 
     if model is None:
         for n in range(1, nModels):
-            atomgroup.addCoordset(coordinates[n*modelSize:(n+1)*modelSize])
+            atomgroup.addCoordset(coordinates[mask][n*modelSize:(n+1)*modelSize])
 
     return atomgroup
+
+
+def writeMMCIF(filename, atoms, csets=None, autoext=True, **kwargs):
+    """Write *atoms* in MMTF format to a file with name *filename* and return
+    *filename*.  If *filename* ends with :file:`.gz`, a compressed file will
+    be written.
+    
+    :arg atoms: an object with atom and coordinate data
+    :type atoms: :class:`.Atomic`
+
+    :arg csets: coordinate set indices, default is all coordinate sets
+
+    :arg autoext: when not present, append extension :file:`.cif` to *filename*
+
+    :keyword header: header to write too
+    :type header: dict
+    """
+    try:
+        from Bio.PDB import MMCIFIO
+    except ImportError:
+        raise ImportError('Biopython MMCIFIO could not be imported. '
+            'Reinstall ProDy or install Biopython '
+            'to solve the problem.')
+
+    header = kwargs.get('header', None)
+
+    if autoext and not filename.lower().endswith('.cif'):
+        filename += '.cif'
+
+    structure = atoms.toBioPythonStructure(header=header, csets=csets)
+    io=MMCIFIO()
+    io.set_structure(structure)
+    io.save(filename)
+    return filename
