@@ -29,6 +29,8 @@ __all__ = ['parsePDBStream', 'parsePDB', 'parseChainsList', 'parsePQR',
 MAX_N_ATOM = 99999 
 MAX_N_RES = 9999
 
+long_id_check_str = "len(pdb) == %d and pdb.startswith('pdb_') and pdb[3] == '_'"
+
 class PDBParseError(Exception):
     pass
 
@@ -64,7 +66,7 @@ _parsePDBdoc = _parsePQRdoc + """
 
     :arg altloc: if a location indicator is passed, such as ``'A'`` or ``'B'``,
          only indicated alternate locations will be parsed as the single
-         coordinate set of the AtomGroup,  if *altloc* is set **True** all
+         coordinate set of the AtomGroup,  if *altloc* is set ``'all'`` then all
          alternate locations will be parsed and each will be appended as a
          distinct coordinate set, default is ``"A"``
     :type altloc: str
@@ -108,6 +110,17 @@ def parsePDB(*pdb, **kwargs):
         Default value is False to reproduce previous behaviour.
         This value is ignored when result is not a list (header=True or model=0).
     :type extend_biomol: bool 
+
+    Please note that resnames are only taken as 3 characters and chids can be 2
+    by default. Hence, TIP3S is split into resname TIP and chid 3S.
+
+    This behaviour can be changed with the following two arguments:
+
+    :arg long_resname: whether to parse longer resnames instead of cutting at 3 chars
+    :type long_resname: bool
+
+    :arg long_chid: whether to parse longer chain ids instead of cutting at 2 chars
+    :type long_chid: bool
     """
     extend_biomol = kwargs.pop('extend_biomol', False)
 
@@ -115,10 +128,9 @@ def parsePDB(*pdb, **kwargs):
     if n_pdb == 0:
         raise ValueError('Please provide a PDB ID or filename')
 
-    if n_pdb == 1:
-        if isListLike(pdb[0]) or isinstance(pdb[0], dict):
-            pdb = pdb[0]
-            n_pdb = len(pdb)
+    if n_pdb == 1 and isListLike(pdb[0]) or isinstance(pdb[0], dict):
+        pdb = pdb[0]
+        n_pdb = len(pdb)
 
     if n_pdb == 1:
         return _parsePDB(pdb[0], **kwargs)
@@ -182,17 +194,17 @@ def parsePDB(*pdb, **kwargs):
 
 def _getPDBid(pdb):
     l = len(pdb)
-    if l == 4:
+    if l == 4 or eval(long_id_check_str % 12):
         pdbid, chain = pdb, ''
-    elif l == 5:
-        pdbid = pdb[:4]; chain = pdb[-1]
+    elif l == 5 or eval(long_id_check_str % 13):
+        pdbid = pdb[:-1]; chain = pdb[-1]
     elif ':' in pdb:
         i = pdb.find(':')
         pdbid = pdb[:i]; chain = pdb[i+1:]
     else:
         raise IOError('{0} is not a valid filename or a valid PDB '
                       'identifier.'.format(pdb))
-    if not pdbid.isalnum():
+    if not (pdbid.isalnum() or len(pdbid) == 12 and pdbid.startswith('pdb_') and pdbid[3] == '_'):
         raise IOError('{0} is not a valid filename or a valid PDB '
                       'identifier.'.format(pdb))
     if chain != '' and not chain.isalnum():
@@ -210,14 +222,17 @@ def _parsePDB(pdb, **kwargs):
         filename = fetchPDB(pdb, **kwargs)
         if filename is None:
             try:
-                LOGGER.warn("Trying to parse mmCIF file instead")
+                if not (eval(long_id_check_str % 12) or eval(long_id_check_str % 13)):
+                    LOGGER.warn("Trying to parse mmCIF file instead")
+                chain = kwargs.pop('chain', chain)
                 return parseMMCIF(pdb+chain, **kwargs)
-            except:
+            except OSError:
                 try:
                     LOGGER.warn("Trying to parse EMD file instead")
+                    chain = kwargs.pop('chain', chain)
                     return parseEMD(pdb+chain, **kwargs)
                 except:
-                    raise IOError('PDB file for {0} could not be downloaded.'
+                    raise IOError('PDB file for {0} could not be parsed.'
                                 .format(pdb))
         pdb = filename
     if title is None:
@@ -270,6 +285,10 @@ def parsePDBStream(stream, **kwargs):
     auto_bonds = SETTINGS.get('auto_bonds')
     get_bonds = kwargs.get('bonds', auto_bonds)
 
+    long_resname = kwargs.get('long_resname')
+    long_chid = kwargs.get('long_chid')
+    strip_icodes = kwargs.get('strip_icodes', True)
+
     if model is not None:
         if isinstance(model, Integral):
             if model < 0:
@@ -321,7 +340,8 @@ def parsePDBStream(stream, **kwargs):
         if header or biomol or secondary:
             hd, split = getHeaderDict(lines)
         bonds = [] if get_bonds else None
-        _parsePDBLines(ag, lines, split, model, chain, subset, altloc, bonds=bonds)
+        _parsePDBLines(ag, lines, split, model, chain, subset, altloc, bonds=bonds, 
+                       long_resname=long_resname, long_chid=long_chid, strip_icodes=strip_icodes)
         if bonds:
             try:
                 ag.setBonds(bonds)
@@ -380,6 +400,9 @@ def parsePQR(filename, **kwargs):
     title = kwargs.get('title', kwargs.get('name'))
     chain = kwargs.get('chain')
     subset = kwargs.get('subset')
+    long_resname = kwargs.get('long_resname')
+    long_chid = kwargs.get('long_chid')
+    strip_icodes = kwargs.get('strip_icodes', True)
     if not os.path.isfile(filename):
         raise IOError('No such file: {0}'.format(repr(filename)))
     if title is None:
@@ -417,7 +440,9 @@ def parsePQR(filename, **kwargs):
     pqr.close()
     LOGGER.timeit()
     ag = _parsePDBLines(ag, lines, split=0, model=1, chain=chain,
-                        subset=subset, altloc_torf=False, format='pqr')
+                        subset=subset, altloc_torf=False, format='pqr', 
+                        long_resname=long_resname, long_chid=long_chid,
+                        strip_icodes=strip_icodes)
     if ag.numAtoms() > 0:
         LOGGER.report('{0} atoms and {1} coordinate sets were '
                       'parsed in %.2fs.'.format(ag.numAtoms(),
@@ -429,7 +454,8 @@ def parsePQR(filename, **kwargs):
 parsePQR.__doc__ += _parsePQRdoc
 
 def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
-                   altloc_torf, format='PDB', bonds=None):
+                   altloc_torf, format='PDB', bonds=None, 
+                   long_resname=False, long_chid=False, strip_icodes=True):
     """Returns an AtomGroup. See also :func:`.parsePDBStream()`.
 
     :arg lines: PDB/PQR lines
@@ -479,11 +505,10 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         elements = np.zeros(asize, dtype=ATOMIC_FIELDS['element'].dtype)
         bfactors = np.zeros(asize, dtype=ATOMIC_FIELDS['beta'].dtype)
         occupancies = np.zeros(asize, dtype=ATOMIC_FIELDS['occupancy'].dtype)
-        anisou = None
         siguij = None
     else:
         radii = np.zeros(asize, dtype=ATOMIC_FIELDS['radius'].dtype)
-
+    anisou = None
     asize = 2000 # increase array length by this much when needed
 
     start = split
@@ -522,28 +547,32 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
     i = start
     END = False
     warned_5_digit = False
+    warned_long_chid = False
     dec = True
     while i < stop:
         line = lines[i]
-        if not isPDB:
-            fields = line.split()
-            if len(fields) == 10:
-                fields.insert(4, '')
-            elif len(fields) != 11:
-                LOGGER.warn('wrong number of fields for PQR format at line %d'%i)
-                i += 1
-                continue
-
-        if isPDB:
-            startswith = line[0:6].strip()
-        else:
-            startswith = fields[0]
+        startswith = line[0:6].strip()
 
         if startswith == 'ATOM' or startswith == 'HETATM':
             if isPDB:
                 atomname = line[12:16].strip()
-                resname = line[17:20].strip()
+
+                if long_resname:
+                    resname = line[17:21].strip()
+                else:
+                    resname = line[17:20].strip()
             else:
+                fields = line.split()
+                if fields[5].find('.') != -1:
+                    # coords too early as no chid
+                    fields.insert(4, '')
+                if len(fields) != 11:
+                    try:
+                        fields = fields[:6] + [line[30:38].strip(), line[38:46].strip(), line[46:54].strip()] + line[54:].split()
+                    except:
+                        LOGGER.warn('wrong number of fields for PQR format at line %d'%i)
+                        i += 1
+                        continue
                 atomname= fields[2]
                 resname = fields[3]
 
@@ -553,7 +582,15 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                     continue
 
             if isPDB:
-                chid = line[20:22].strip()
+                if long_chid:
+                    chid = line[20:22].strip()
+                    if len(chid) > 1 and not warned_long_chid:
+                        LOGGER.warn('Parsed 2-character chid {0} continuous with resnum {1} from {2}. Please check if this was intended.'.format(
+                            chid, line[17:20], line[17:22]
+                        ))
+                        warned_long_chid = True
+                else:
+                    chid = line[21].strip()
             else:
                 chid = fields[4]
 
@@ -606,7 +643,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                 serials[acount] = int(serial_str)
             except ValueError:
                 try:
-                    isnumeric = np.alltrue([x.isdigit() for x in serial_str])
+                    isnumeric = np.all([x.isdigit() for x in serial_str])
                     if not isnumeric and serial_str == serial_str.upper():
                         serials[acount] = hybrid36ToDec(serial_str)
                     else:
@@ -649,7 +686,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                 if not dec:
                     resnum = resnum_str
                     try:
-                        isnumeric = np.alltrue([x.isdigit() or x==' ' for x in resnum_str])
+                        isnumeric = np.all([x.isdigit() or x==' ' for x in resnum_str])
                         if not isnumeric and resnum_str == resnum_str.upper():
                             resnum = hybrid36ToDec(resnum_str, resnum=True)
                         else:
@@ -745,8 +782,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                     elements = np.concatenate((elements,
                         np.zeros(asize, ATOMIC_FIELDS['element'].dtype)))
                     if anisou is not None:
-                        anisou = np.concatenate((anisou, np.zeros((asize, 6),
-                            ATOMIC_FIELDS['anisou'].dtype)))
+                        anisou = np.concatenate((anisou, np.zeros((asize, 6), float)))
                     if siguij is not None:
                         siguij = np.concatenate((siguij, np.zeros((asize, 6),
                             ATOMIC_FIELDS['siguij'].dtype)))
@@ -816,9 +852,12 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                 if END:
                     coordinates.resize((acount, 3), refcheck=False)
                     if addcoords:
-                        atomgroup.addCoordset(coordinates)
+                        atomgroup.addCoordset(coordinates, anisous=anisou)
                     else:
                         atomgroup._setCoords(coordinates)
+                        if isPDB and anisou is not None:
+                            anisou.resize((acount, 6), refcheck=False)
+                            atomgroup.setAnisous(anisou / 10000)
                 else:
                     coordsets = np.zeros((int(diff//acount+1), acount, 3))
                     coordsets[0] = coordinates[:acount]
@@ -843,7 +882,9 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                 atomgroup.setFlags('pdbter', termini)
                 atomgroup.setFlags('selpdbter', termini)
                 atomgroup.setAltlocs(altlocs)
-                atomgroup.setIcodes(np.char.strip(icodes))
+                if strip_icodes:
+                    icodes = np.char.strip(icodes)
+                atomgroup.setIcodes(icodes)
                 atomgroup.setSerials(serials)
                 if isPDB:
                     bfactors.resize(acount, refcheck=False)
@@ -881,8 +922,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         elif isPDB and startswith == 'ANISOU':
             if anisou is None:
                 anisou = True
-                anisou = np.zeros((alength, 6),
-                    dtype=ATOMIC_FIELDS['anisou'].dtype)
+                anisou = np.zeros((alength, 6), dtype=float)
 
             alt = line[16]
             if alt not in which_altlocs and which_altlocs != 'all':
@@ -932,7 +972,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         # this means last line was an ATOM line, so atomgroup is not decorated
         coordinates.resize((acount, 3), refcheck=False)
         if addcoords:
-            atomgroup.addCoordset(coordinates)
+            atomgroup.addCoordset(coordinates, anisous=anisou)
         else:
             atomgroup._setCoords(coordinates)
         atomnames.resize(acount, refcheck=False)
@@ -955,7 +995,9 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         atomgroup.setFlags('pdbter', termini)
         atomgroup.setFlags('selpdbter', termini)
         atomgroup.setAltlocs(altlocs)
-        atomgroup.setIcodes(np.char.strip(icodes))
+        if strip_icodes:
+            icodes = np.char.strip(icodes)
+        atomgroup.setIcodes(icodes)
         atomgroup.setSerials(serials)
         if isPDB:
             if anisou is not None:
@@ -1002,6 +1044,7 @@ def _evalAltlocs(atomgroup, altloc, chainids, resnums, resnames, atomnames):
     indices = {}
     for key in altloc_keys:
         xyz = atomgroup.getCoords()
+        anisou = atomgroup.getAnisous()
         success = 0
         lines = altloc[key]
         for line, i in lines:
@@ -1028,7 +1071,7 @@ def _evalAltlocs(atomgroup, altloc, chainids, resnums, resnames, atomnames):
                 LOGGER.warn("failed to parse altloc {0} at line {1}, "
                             "residue name mismatch (expected {2}, "
                             "parsed {3})".format(repr(key), i+1, repr(rn),
-                                                   repr(arn)))
+                                                 repr(arn)))
                 continue
             index = ids[(ans == aan).nonzero()[0]]
             if len(index) != 1:
@@ -1036,21 +1079,35 @@ def _evalAltlocs(atomgroup, altloc, chainids, resnums, resnames, atomnames):
                             " {2} not found in the residue"
                             .format(repr(key), i+1, repr(aan)))
                 continue
-            try:
-                xyz[index[0], 0] = float(line[30:38])
-                xyz[index[0], 1] = float(line[38:46])
-                xyz[index[0], 2] = float(line[46:54])
-            except:
-                LOGGER.warn('failed to parse altloc {0} at line {1}, could'
-                            ' not read coordinates'.format(repr(key), i+1))
-                continue
-            success += 1
+
+            if line.startswith('ATOM  '):
+                try:
+                    xyz[index[0], 0] = float(line[30:38])
+                    xyz[index[0], 1] = float(line[38:46])
+                    xyz[index[0], 2] = float(line[46:54])
+                except:
+                    LOGGER.warn('failed to parse altloc {0} at line {1}, could'
+                                ' not read coordinates'.format(repr(key), i+1))
+                    continue
+                success += 1
+            elif line.startswith('ANISOU'):
+                try:
+                    anisou[index[0], 0] = line[28:35]
+                    anisou[index[0], 1] = line[35:42]
+                    anisou[index[0], 2] = line[43:49]
+                    anisou[index[0], 3] = line[49:56]
+                    anisou[index[0], 4] = line[56:63]
+                    anisou[index[0], 5] = line[63:70]
+                except:
+                    LOGGER.warn('failed to parse altloc {0} at line {1}, could'
+                                ' not read anisous'.format(repr(key), i+1))
+                success += 1
         LOGGER.info('{0} out of {1} altloc {2} lines were parsed.'
                     .format(success, len(lines), repr(key)))
         if success > 0:
             LOGGER.info('Altloc {0} is appended as a coordinate set to '
                         'atomgroup {1}.'.format(repr(key), atomgroup.getTitle()))
-            atomgroup.addCoordset(xyz, label='altloc ' + key)
+            atomgroup.addCoordset(xyz, label='altloc ' + key, anisous=anisou)
 
 
 #HELIXLINE = ('HELIX  %3d %3s %-3s %1s %4d%1s %-3s %1s %4d%1s%2d'
@@ -1202,9 +1259,19 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
         Default is **False**, which means using hexadecimal instead.
         NB: ChimeraX seems to prefer hybrid36 and may have problems with hexadecimal.
     :type hybrid36: bool
-    """
 
+    :arg full_ter: whether to write full TER lines with atoms info
+        Default is **True**
+    :type full_ter: bool
+
+    :arg write_remarks: whether to write REMARK lines
+        Default is **True**
+    :type write_remarks: bool
+    """    
     renumber = kwargs.get('renumber', True)
+    full_ter = kwargs.get('full_ter', True)
+    write_remarks = kwargs.get('write_remarks', True)
+    long_resname = kwargs.get('long_resname', False)
 
     remark = str(atoms)
     try:
@@ -1222,8 +1289,10 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
     if coordsets is None:
         raise ValueError('atoms does not have any coordinate sets')
 
+    had_atoms = False
     try:
         acsi = atoms.getACSIndex()
+        had_atoms = True
     except AttributeError:
         try:
             atoms = atoms.getAtoms()
@@ -1335,72 +1404,76 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
             if charges2[i] == '0+':
                 charges2[i] = '  '
 
-    anisous = atoms._getAnisous()
-    if anisous is not None:
-        anisous = np.array(anisous * 10000, dtype=int)
+    if write_remarks:
+        # write remarks
+        stream.write('REMARK {0}\n'.format(remark))
 
-    # write remarks
-    stream.write('REMARK {0}\n'.format(remark))
+        # write secondary structures (if any)
+        secondary = kwargs.get('secondary', True)
+        secstrs = atoms._getSecstrs()
+        if secstrs is not None and secondary:
+            secindices = atoms._getSecindices()
+            secclasses = atoms._getSecclasses()
+            secids = atoms._getSecids()
 
-    # write secondary structures (if any)
-    secondary = kwargs.get('secondary', True)
-    secstrs = atoms._getSecstrs()
-    if secstrs is not None and secondary:
-        secindices = atoms._getSecindices()
-        secclasses = atoms._getSecclasses()
-        secids = atoms._getSecids()
+            # write helices
+            for i in range(1,max(secindices)+1):
+                torf = np.logical_and(isHelix(secstrs), secindices==i)
+                if torf.any():
+                    helix_resnums = resnums[torf]
+                    helix_chainids = chainids[torf]
+                    helix_resnames = resnames[torf]
+                    helix_secclasses = secclasses[torf]
+                    helix_secids = secids[torf]
+                    helix_icodes = icodes[torf]
+                    L = helix_resnums[-1] - helix_resnums[0] + 1
 
-        # write helices
-        for i in range(1,max(secindices)+1):
-            torf = np.logical_and(isHelix(secstrs), secindices==i)
-            if torf.any():
-                helix_resnums = resnums[torf]
-                helix_chainids = chainids[torf]
-                helix_resnames = resnames[torf]
-                helix_secclasses = secclasses[torf]
-                helix_secids = secids[torf]
-                helix_icodes = icodes[torf]
-                L = helix_resnums[-1] - helix_resnums[0] + 1
+                    stream.write(HELIXLINE.format(serNum=i, helixID=helix_secids[0], 
+                                initResName=helix_resnames[0][:3], initChainID=helix_chainids[0], 
+                                initSeqNum=helix_resnums[0], initICode=helix_icodes[0],
+                                endResName=helix_resnames[-1][:3], endChainID=helix_chainids[-1], 
+                                endSeqNum=helix_resnums[-1], endICode=helix_icodes[-1],
+                                helixClass=helix_secclasses[0], length=L))
 
-                stream.write(HELIXLINE.format(serNum=i, helixID=helix_secids[0], 
-                            initResName=helix_resnames[0], initChainID=helix_chainids[0], 
-                            initSeqNum=helix_resnums[0], initICode=helix_icodes[0],
-                            endResName=helix_resnames[-1], endChainID=helix_chainids[-1], 
-                            endSeqNum=helix_resnums[-1], endICode=helix_icodes[-1],
-                            helixClass=helix_secclasses[0], length=L))
+            # write strands
+            torf_all_sheets = isSheet(secstrs)
+            sheet_secids = secids[torf_all_sheets]
 
-        # write strands
-        torf_all_sheets = isSheet(secstrs)
-        sheet_secids = secids[torf_all_sheets]
+            unique_sheet_secids, indices = np.unique(sheet_secids, return_index=True)
+            unique_sheet_secids = unique_sheet_secids[indices.argsort()]
+            for sheet_id in unique_sheet_secids:
+                torf_strands_in_sheet = np.logical_and(torf_all_sheets, secids==sheet_id)
+                strand_indices = secindices[torf_strands_in_sheet]
+                numStrands = len(np.unique(strand_indices))
 
-        unique_sheet_secids, indices = np.unique(sheet_secids, return_index=True)
-        unique_sheet_secids = unique_sheet_secids[indices.argsort()]
-        for sheet_id in unique_sheet_secids:
-            torf_strands_in_sheet = np.logical_and(torf_all_sheets, secids==sheet_id)
-            strand_indices = secindices[torf_strands_in_sheet]
-            numStrands = len(np.unique(strand_indices))
+                for i in np.unique(strand_indices):
+                    torf_strand = np.logical_and(torf_strands_in_sheet, secindices==i)
+                    strand_resnums = resnums[torf_strand]
+                    strand_chainids = chainids[torf_strand]
+                    strand_resnames = resnames[torf_strand]
+                    strand_secclasses = secclasses[torf_strand]
+                    strand_icodes = icodes[torf_strand]
 
-            for i in np.unique(strand_indices):
-                torf_strand = np.logical_and(torf_strands_in_sheet, secindices==i)
-                strand_resnums = resnums[torf_strand]
-                strand_chainids = chainids[torf_strand]
-                strand_resnames = resnames[torf_strand]
-                strand_secclasses = secclasses[torf_strand]
-                strand_icodes = icodes[torf_strand]
-
-                stream.write(SHEETLINE.format(strand=i, sheetID=sheet_id, numStrands=numStrands,
-                            initResName=strand_resnames[0], initChainID=strand_chainids[0], 
-                            initSeqNum=strand_resnums[0], initICode=strand_icodes[0],
-                            endResName=strand_resnames[-1], endChainID=strand_chainids[-1], 
-                            endSeqNum=strand_resnums[-1], endICode=strand_icodes[-1],
-                            sense=strand_secclasses[0]))
-        pass
+                    stream.write(SHEETLINE.format(strand=i, sheetID=sheet_id, numStrands=numStrands,
+                                initResName=strand_resnames[0][:3], initChainID=strand_chainids[0], 
+                                initSeqNum=strand_resnums[0], initICode=strand_icodes[0],
+                                endResName=strand_resnames[-1][:3], endChainID=strand_chainids[-1], 
+                                endSeqNum=strand_resnums[-1], endICode=strand_icodes[-1],
+                                sense=strand_secclasses[0]))
+            pass
 
     # write atoms
     multi = len(coordsets) > 1
     write = stream.write
     num_ter_lines = 0
     for m, coords in enumerate(coordsets):
+
+        if had_atoms:
+            atoms.setACSIndex(m)
+        anisous = atoms._getAnisous()
+        if anisous is not None:
+            anisous = np.array(anisous * 10000, dtype=int)
+
         if multi:
             write('MODEL{0:9d}\n'.format(m+1))
 
@@ -1415,6 +1488,7 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
             warned_hybrid36 = False
 
         warned_5_digit = False
+        warned_long_resname = False
 
         for i, xyz in enumerate(coords):
 
@@ -1457,9 +1531,22 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
                 serial = decToHybrid36(serial)
                 resnum = decToHybrid36(resnums[i], resnum=True)
             else:
-                serial = serial
                 resnum = resnums[i]
 
+            resname = resnames[i]
+            resname_len = 3
+            if long_resname:
+                resname_len = 4
+                pdbline = pdbline.replace('%-3s%2s', '%-4s%1s')
+
+            if len(resnames[i]) > resname_len:
+                resname = resname[:resname_len]
+                if not warned_long_resname:
+                    LOGGER.warn('Resname {0} too long, cutting resname to {1} characters as {2}'.format(
+                        resnames[i], resname_len, resname
+                    ))
+                    warned_long_resname = True
+
             if pdbline == PDBLINE_LT100K or hybrid36:
                 if len(str(resnum)) == 5:
                     if icodes[i] == '':
@@ -1507,7 +1594,7 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
 
             write(pdbline % (hetero[i], serial,
                              atomnames[i], altlocs[i],
-                             resnames[i], chainids[i], resnum,
+                             resname, chainids[i][:1], resnum,
                              icodes[i],
                              xyz[0], xyz[1], xyz[2],
                              occupancies[i], bfactors[i],
@@ -1518,7 +1605,7 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
 
                 write(anisouline % ("ANISOU", serial,
                                     atomnames[i], altlocs[i],
-                                    resnames[i], chainids[i], resnum,
+                                    resname, chainids[i], resnum,
                                     icodes[i],
                                     anisou[0], anisou[1], anisou[2],
                                     anisou[3], anisou[4], anisou[5],
@@ -1531,13 +1618,16 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
                     else:
                         serial += 1
 
-                    false_pdbline = pdbline % ("TER   ", serial,
-                                               "", "",
-                                               resnames[i], chainids[i], resnum,
-                                               icodes[i],
-                                               xyz[0], xyz[1], xyz[2],
-                                               occupancies[i], bfactors[i],
-                                               segments[i], elements[i], charges2[i])
+                    if full_ter:
+                        false_pdbline = pdbline % ("TER   ", serial,
+                                                   "", "",
+                                                   resname, chainids[i], resnum,
+                                                   icodes[i],
+                                                   xyz[0], xyz[1], xyz[2],
+                                                   occupancies[i], bfactors[i],
+                                                   segments[i], elements[i], charges2[i])
+                    else:
+                        false_pdbline = "TER" + " "*23
                     write(false_pdbline[:26] + " "*54 + '\n')
                     num_ter_lines += 1
             else:
@@ -1547,13 +1637,16 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
                     else:
                         serial += 1
 
-                    false_pdbline = pdbline % ("TER   ", serial,
-                                               "", "",
-                                               resnames[i], chainids[i], resnum,
-                                               icodes[i],
-                                               xyz[0], xyz[1], xyz[2],
-                                               occupancies[i], bfactors[i],
-                                               segments[i], elements[i], charges2[i])
+                    if full_ter:
+                        false_pdbline = pdbline % ("TER   ", serial,
+                                                   "", "",
+                                                   resname, chainids[i], resnum,
+                                                   icodes[i],
+                                                   xyz[0], xyz[1], xyz[2],
+                                                   occupancies[i], bfactors[i],
+                                                   segments[i], elements[i], charges2[i])
+                    else:
+                        false_pdbline = "TER" + " "*23
                     write(false_pdbline[:26] + " "*54 + '\n')
                     num_ter_lines += 1
 
@@ -1563,6 +1656,9 @@ def writePDBStream(stream, atoms, csets=None, **kwargs):
             
     write('END   ' + " "*74 + '\n')
 
+    if had_atoms:
+        atoms.setACSIndex(acsi)
+
 writePDBStream.__doc__ += _writePDBdoc
 
 def writePDB(filename, atoms, csets=None, autoext=True, **kwargs):
@@ -1578,10 +1674,16 @@ def writePDB(filename, atoms, csets=None, autoext=True, **kwargs):
         Default is **False**, which means using hexadecimal instead.
         NB: ChimeraX seems to prefer hybrid36 and may have problems with hexadecimal.
     :type hybrid36: bool
+
+    Please note that resnames longer than 3 characters will be trimmed by default.
+    This behaviour can be changed with the following argument:
+
+    :arg long_resname: whether to write 4-character resnames instead of cutting at 3 chars
+    :type long_resname: bool
     """
 
     if not (filename.lower().endswith('.pdb') or filename.lower().endswith('.pdb.gz') or
-            filename.lower().endswith('.ent') or filename.lower().endswith('.ent.gz')):
+            filename.lower().endswith('.ent') or filename.lower().endswith('.ent.gz')) and autoext:
         filename += '.pdb'
     out = openFile(filename, 'wt')
     writePDBStream(out, atoms, csets, **kwargs)
@@ -1614,7 +1716,10 @@ def writePQRStream(stream, atoms, **kwargs):
     write = stream.write
 
     calphas = atoms.ca
-    ssa = calphas.getSecstrs()
+    if calphas is not None:
+        ssa = calphas.getSecstrs()
+    else:
+        ssa = None
     helix = []
     sheet = []
     if ssa is not None:
