@@ -1026,12 +1026,17 @@ def parseImagesFromSTAR(particlesSTAR, **kwargs):
     return np.array(images), parsed_images_data
 
 
-def parseSTARSection(lines, key):
+def parseSTARSection(lines, key, report=True):
     """Parse a section of data from *lines* from a STAR file 
     corresponding to a *key* (part before the dot). 
     This can be a loop or data block.
     
-    Returns data encapulated in a list and the associated fields."""
+    Returns data encapulated in a list and the associated fields.
+    
+    :arg report: whether to report warnings about not finding data
+        default True
+    :type report: bool
+    """
 
     if not isinstance(key, str):
         raise TypeError("key should be a string")
@@ -1068,8 +1073,8 @@ def parseSTARSection(lines, key):
 
         i += 1
 
-    if i < len(lines):
-        star_dict, _ = parseSTARLines(lines[:2] + lines[start-1: stop], shlex=True)
+    if i < len(lines) or start > 0:
+        star_dict, _ = parseSTARLines(lines[:5] + lines[start-1: stop], shlex=True)
         loop_dict = list(star_dict.values())[0]
 
         if lines[start - 1].strip() == "loop_":
@@ -1077,7 +1082,8 @@ def parseSTARSection(lines, key):
         else:
             data = [loop_dict["data"]]
     else:
-        LOGGER.warn("Could not find {0} in lines.".format(key))
+        if report:
+            LOGGER.warn("Could not find {0} in lines.".format(key))
         return []
 
     return data
