@@ -1,38 +1,59 @@
 # -*- coding: utf-8 -*-
 
-"""This module detects, predicts and analyzes water bridges.
+"""This module provides the the WatFinder toolkit that detects, predicts and analyzes water bridges.
 """
 
 __author__ = 'Karolina Mikulska-Ruminska'
 __credits__ = ['Frane Doljanin', 'Karolina Mikulska-Ruminska']
 __email__ = ['karolamik@fizyka.umk.pl', 'fdoljanin@pmfst.hr']
 
+import multiprocessing as mp
+
+from numbers import Number
 import numpy as np
+import os
 
 from itertools import combinations
 from collections import deque
-from enum import Enum, auto
+
+from prody import PY3K
+if PY3K:
+    from enum import Enum, auto
+else:
+    from enum import Enum
+    import enum
+    from itertools import count
+    def auto(it=count()):
+        return next(it)
+    enum.auto=auto
+
 from copy import copy
 
-from prody import LOGGER
+from prody import LOGGER, SETTINGS
 from prody.atomic import Atom, Atomic, AtomGroup
-from prody.dynamics.plotting import showAtomicMatrix
+from prody.atomic.flags import DEFAULTS
 from prody.ensemble import Ensemble
 from prody.measure import calcAngle, calcDistance
 from prody.measure.contacts import findNeighbors
-from prody.proteins import writePDB
+from prody.proteins import writePDB, parsePDB
+
+from prody.utilities import showFigure, showMatrix
 
 
 __all__ = ['calcWaterBridges', 'calcWaterBridgesTrajectory', 'getWaterBridgesInfoOutput',
-           'calcWaterBridgesStatistics', 'getWaterBridgeStatInfo', 'calcWaterBridgeMatrix', 'showWaterBridgeMatrix',
+           'calcWaterBridgesStatistics', 'getWaterBridgeStatInfo',
+           'calcWaterBridgeMatrix', 'showWaterBridgeMatrix',
            'calcBridgingResiduesHistogram', 'calcWaterBridgesDistribution',
            'savePDBWaterBridges', 'savePDBWaterBridgesTrajectory',
-           'saveWaterBridges', 'parseWaterBridges']
+           'saveWaterBridges', 'parseWaterBridges', 'findClusterCenters',
+           'filterStructuresWithoutWater', 'selectSurroundingsBox',
+           'findCommonSelectionTraj']
 
 
 class ResType(Enum):
     WATER = auto()
     PROTEIN = auto()
+    ION = auto()
 
 
 class AtomNode:
@@ -110,6 +131,8 @@ class HydrogenBond:
 
     @staticmethod
     def checkIsHBond(donor, acceptor, constraints):
+        if donor.type == ResType.ION or acceptor.type == ResType.ION:
+            return False
         if donor.type != ResType.WATER and donor.atom.getName()[0] not in constraints.donors:
             return False
         if acceptor.type != ResType.WATER and acceptor.atom.getName()[0] not in constraints.acceptors:
@@ -136,6 +159,22 @@ class HydrogenBond:
 
         return False
 
+class CoordinationBond:
+    def __init__(self, donor, acceptor):
+        self.donor = donor
+        self.acceptor = acceptor
+
+    @staticmethod
+    def checkIsCoorBond(donor, acceptor, constraints):
+        if donor.type != ResType.WATER and donor.atom.getName() not in constraints.donors:
+            return False
+        if acceptor.type != ResType.WATER and acceptor.atom.getName() not in constraints.acceptors:
+            return False
+        if donor.type != ResType.ION and acceptor.type != ResType.ION:
+            return False
+
+        return True
+
 
 def calcBridges(relations, hydrophilicList, method, maxDepth, maxNumResidues):
     waterBridges = []
@@ -256,8 +295,10 @@ def getInfoOutput(waterBridgesAtomic):
         bridgeOutput = []
 
         for atom in bridge.proteins:
-            residueInfo = f"{atom.getResname()}{atom.getResnum()}"
-            atomInfo = f"{atom.getName()}_{atom.getIndex()}"
+            residueInfo = "{0}{1}".format(atom.getResname(), 
+                                          atom.getResnum())
+            atomInfo = "{0}_{1}".format(atom.getName(),
+                                        atom.getIndex())
             chainInfo = atom.getChid()
             bridgeOutput += [residueInfo, atomInfo, chainInfo]
 
@@ -266,7 +307,9 @@ def getInfoOutput(waterBridgesAtomic):
 
         bridgeOutput += [len(bridge.waters)]
         bridgeOutput += [
-            list(map(lambda w: f"{w.getChid()}_{w.getIndex()}", bridge.waters))]
+            list(map(lambda w: "{0}_{1}".format(w.getChid(),
+                                                w.getIndex()), 
+                     bridge.waters))]
 
         output.append(bridgeOutput)
 
@@ -296,7 +339,7 @@ def getAtomicOutput(waterBridges, relations):
     for bridge in waterBridges:
         proteinAtoms, waterAtoms = [], []
         for atomIndex in bridge:
-            if relations[atomIndex].type == ResType.PROTEIN:
+            if relations[atomIndex].type in [ResType.PROTEIN, ResType.ION]:
                 proteinAtoms.append(relations[atomIndex].atom)
             else:
                 waterAtoms.append(relations[atomIndex].atom)
@@ -307,7 +350,7 @@ def getAtomicOutput(waterBridges, relations):
 
 
 def getElementsRegex(elements):
-    return f'[{"|".join(elements)}].*'
+    return '[{0}].*'.format("|".join(elements))
 
 
 def calcWaterBridges(atoms, **kwargs):
@@ -363,6 +406,19 @@ def calcWaterBridges(atoms, **kwargs):
     :arg isInfoLog: should log information
         default is True
     :type output: bool
+
+    :arg selstr: selection string for focusing analysis
+        default of **None** focuses on everything
+    :type selstr: str
+
+    :arg expand_selection: whether to expand the selection with 
+        :func:`.selectSurroundingsBox`, selecting a box surrounding it.
+        Default is **False**
+    :type expand_selection: bool
+
+    :arg considered_atoms_sel: selection string for which atoms to consider
+        Default is **"protein"**
+    :type considered_atoms_sel: str
     """
 
     method = kwargs.pop('method', 'chain')
@@ -378,20 +434,40 @@ def calcWaterBridges(atoms, **kwargs):
     outputType = kwargs.pop('output', 'atomic')
     isInfoLog = kwargs.pop('isInfoLog', True)
     DIST_COVALENT_H = 1.4
+    prefix = kwargs.pop('prefix', '')
+    considered_atoms_sel = kwargs.pop('considered_atoms_sel', "protein")
 
     if method not in ['chain', 'cluster']:
         raise TypeError('Method should be chain or cluster.')
     if outputType not in ['info', 'atomic', 'indices']:
         raise TypeError('Output can be info, atomic or indices.')
 
+    selstr = kwargs.pop('selstr', None)
+    if selstr is not None:
+        selection = atoms.select(selstr).copy()
+
+        expand_selection = kwargs.pop('expand_selection', False)
+        if expand_selection:
+            atoms = selectSurroundingsBox(atoms, selection, **kwargs).copy()
+        else:
+            atoms = selection.copy()
+
+    water = atoms.select('water')
+    if water is None:
+        raise ValueError('atoms has no water so cannot be analysed with WatFinder')
+
     relations = RelationList(len(atoms))
-    consideredAtoms = ~atoms.select(
-        f'water and not within {distWR} of protein')
+    tooFarAtoms = atoms.select(
+        'water and not within {0} of protein'.format(distWR))
+    if tooFarAtoms is None:
+        consideredAtoms = atoms
+    else:
+        consideredAtoms = ~tooFarAtoms
 
     waterHydrogens = consideredAtoms.select('water and hydrogen') or []
     waterOxygens = consideredAtoms.select('water and oxygen')
     waterHydroOxyPairs = findNeighbors(
-        waterOxygens, DIST_COVALENT_H, waterHydrogens)
+        waterOxygens, DIST_COVALENT_H, waterHydrogens) if waterHydrogens else []
     for oxygen in waterOxygens:
         relations.addNode(oxygen, ResType.WATER)
     for pair in waterHydroOxyPairs:
@@ -399,13 +475,18 @@ def calcWaterBridges(atoms, **kwargs):
         relations[oxygen].hydrogens.append(hydrogen)
 
     proteinHydrophilic = consideredAtoms.select(
-        f'protein and name "{getElementsRegex(set(donors+acceptors))}" and within {distWR} of water')
+        '({0}) and name "{1}" and within {2} of water'.format(considered_atoms_sel,
+            getElementsRegex(set(donors+acceptors)), distWR))
 
-    proteinHydrogens = consideredAtoms.select(f'protein and hydrogen') or []
+    proteinHydrogens = consideredAtoms.select('({0}) and hydrogen'.format(considered_atoms_sel)) or []
     proteinHydroPairs = findNeighbors(
-        proteinHydrophilic, DIST_COVALENT_H, proteinHydrogens)
+        proteinHydrophilic, DIST_COVALENT_H, proteinHydrogens) if proteinHydrogens else []
     for hydrophilic in proteinHydrophilic:
-        relations.addNode(hydrophilic, ResType.PROTEIN)
+        if hydrophilic is not None:
+            if hydrophilic.getFlag('ion'):
+                relations.addNode(hydrophilic, ResType.ION)
+            else:
+                relations.addNode(hydrophilic, ResType.PROTEIN)
     for pair in proteinHydroPairs:
         hydrophilic, hydrogen, _ = pair
         relations[hydrophilic].hydrogens.append(hydrogen)
@@ -425,12 +506,18 @@ def calcWaterBridges(atoms, **kwargs):
     else:
         LOGGER.info('No hydrogens detected, angle criteria will not be used.')
 
+    constraints2 = HBondConstraints(acceptors, donors) # not taking angles
+
     for pair in contactingWaterNodes + contactingWaterProteinNodes:
         for a, b in [(0, 1), (1, 0)]:
             if HydrogenBond.checkIsHBond(pair[a], pair[b], constraints):
                 newHBond = HydrogenBond(pair[a].atom, pair[b].atom)
                 relations.addHBond(newHBond)
 
+            elif CoordinationBond.checkIsCoorBond(pair[a], pair[b], constraints2):
+                newBond = CoordinationBond(pair[a].atom, pair[b].atom)
+                relations.addHBond(newBond)
+
     relations.removeUnbonded()
 
     waterBridgesWithIndices = calcBridges(
@@ -439,8 +526,12 @@ def calcWaterBridges(atoms, **kwargs):
         waterBridgesWithIndices = getUniqueElements(
             waterBridgesWithIndices, getChainBridgeTuple)
 
-    LOGGER.info(
-        f'{len(waterBridgesWithIndices)} water bridges detected.')
+    log_string = '{0} water bridges detected using method {1}'.format(
+        len(waterBridgesWithIndices), method)
+    if prefix != '':
+        log_string += ' for ' + prefix
+    LOGGER.info(log_string)
+
     if method == 'atomic':
         LOGGER.info('Call getInfoOutput to convert atomic to info output.')
 
@@ -466,18 +557,47 @@ def calcWaterBridgesTrajectory(atoms, trajectory, **kwargs):
     :arg atoms: Atomic object from which atoms are considered
     :type atoms: :class:`.Atomic`
 
-    :arg trajectory: trajectory object, DCD or multimodal PDB
+    :arg trajectory: Trajectory data coming from a DCD, ensemble or multi-model PDB file.
+    :type trajectory: :class:`.Trajectory', :class:`.Ensemble`, :class:`.Atomic`
 
     :arg start_frame: frame to start from
     :type start_frame: int
 
     :arg stop_frame: frame to stop
     :type stop_frame: int
-    """
 
-    interactions_all = []
+    :arg max_proc: maximum number of processes to use
+        default is half of the number of CPUs
+    :type max_proc: int
+
+    :arg selstr: selection string for focusing analysis
+        default of **None** focuses on everything
+    :type selstr: str
+
+    :arg expand_selection: whether to expand the selection with 
+        :func:`.selectSurroundingsBox`, selecting a box surrounding it.
+        Default is **False**
+    :type expand_selection: bool
+
+    If selstr is provided, a common selection will be found across all frames
+    combining selections satifying the criteria in each.
+
+    :arg return_selection: whether to return the combined common selection
+        Default is **False** to keep expected behaviour.
+        However, this output is required when using selstr.
+    :type return_selection: bool
+
+    :arg considered_atoms_sel: selection string for which atoms to consider
+        Default is **"protein"**
+    :type considered_atoms_sel: str  
+    """
     start_frame = kwargs.pop('start_frame', 0)
     stop_frame = kwargs.pop('stop_frame', -1)
+    max_proc = kwargs.pop('max_proc', mp.cpu_count()//2)
+    selstr = kwargs.pop('selstr', None)
+    expand_selection = kwargs.pop('expand_selection', False)
+    return_selection = kwargs.pop('return_selection', False)
+    padding = kwargs.pop('padding', 0)
 
     if trajectory is not None:
         if isinstance(trajectory, Atomic):
@@ -485,7 +605,7 @@ def calcWaterBridgesTrajectory(atoms, trajectory, **kwargs):
 
         # nfi = trajectory._nfi
         # trajectory.reset()
-        numFrames = trajectory._n_csets
+        # numFrames = trajectory._n_csets
 
         if stop_frame == -1:
             traj = trajectory[start_frame:]
@@ -493,29 +613,121 @@ def calcWaterBridgesTrajectory(atoms, trajectory, **kwargs):
             traj = trajectory[start_frame:stop_frame+1]
 
         atoms_copy = atoms.copy()
-        for j0, frame0 in enumerate(traj, start=start_frame):
+        if selstr is not None:
+            selection, indices = findCommonSelectionTraj(atoms, traj, selstr,
+                                                         expand_selection=expand_selection,
+                                                         return_selection=False,
+                                                         padding=padding)
+            LOGGER.info('Common selection found with {0} atoms and {1} protein chains'.format(
+                selection.numAtoms(), len(list(selection.protein.getHierView()))))
+
+        def analyseFrame(j0, start_frame, frame0, interactions_all):
             LOGGER.info('Frame: {0}'.format(j0))
+            atoms_copy = atoms.copy()
             atoms_copy.setCoords(frame0.getCoords())
+
+            if indices is not None:
+                atoms_copy = atoms_copy[indices]
+                kwargs['selstr'] = atoms_copy.getSelstr()
+
             interactions = calcWaterBridges(
-                atoms_copy, isInfoLog=False, **kwargs)
-            interactions_all.append(interactions)
+                atoms_copy, isInfoLog=False, 
+                prefix='frame {0}'.format(j0),
+                **kwargs)
+            interactions_all[j0-start_frame] = interactions
+
+        if max_proc == 1:
+            interactions_all = []
+            for j0, frame0 in enumerate(traj, start=start_frame):
+                interactions_all.append([])
+                analyseFrame(j0, start_frame, frame0, interactions_all)
+        else:
+            with mp.Manager() as manager:
+                interactions_all = manager.list()
+                for j0, frame0 in enumerate(traj, start=start_frame):
+                    interactions_all.append([])
+
+                j0 = start_frame
+                while j0 < traj.numConfs()+start_frame:
+
+                    processes = []
+                    for _ in range(max_proc):
+                        frame0 = traj[j0-start_frame]
+                        
+                        p = mp.Process(target=analyseFrame, args=(j0, start_frame,
+                                                                 frame0,
+                                                                 interactions_all))
+                        p.start()
+                        processes.append(p)
+
+                        j0 += 1
+                        if j0 >= traj.numConfs()+start_frame:
+                            break
+
+                    for p in processes:
+                        p.join()
+
+                interactions_all = interactions_all[:]
+
         # trajectory._nfi = nfi
 
     else:
         if atoms.numCoordsets() > 1:
-            for i in range(len(atoms.getCoordsets()[start_frame:stop_frame])):
-                LOGGER.info('Model: {0}'.format(i+start_frame))
+            def analyseFrame(i, interactions_all):
+                frameNum = i+start_frame
+                LOGGER.info('Model: {0}'.format(frameNum))
                 atoms.setACSIndex(i+start_frame)
-                interactions = calcWaterBridges(atoms, **kwargs)
-                interactions_all.append(interactions)
+                interactions = calcWaterBridges(
+                    atoms, isInfoLog=False, prefix='frame {0}'.format(frameNum),
+                    **kwargs)
+                interactions_all[i] = interactions
+
+            if max_proc == 1:
+                interactions_all = []
+                for i in range(len(atoms.getCoordsets()[start_frame:stop_frame])):
+                    interactions_all.append([])
+                    analyseFrame(i, interactions_all)
+            else:
+                with mp.Manager() as manager:
+                    interactions_all = manager.list()
+                    for i in range(len(atoms.getCoordsets()[start_frame:stop_frame])):
+                        interactions_all.append([])
+
+                    i = start_frame
+                    while i < len(atoms.getCoordsets()[start_frame:stop_frame]):
+                        processes = []
+                        for _ in range(max_proc):
+                            p = mp.Process(target=analyseFrame, args=(i, interactions_all))
+                            p.start()
+                            processes.append(p)
+
+                            i += 1
+                            if i >= len(atoms.getCoordsets()[start_frame:stop_frame]):
+                                break
+
+                        for p in processes:
+                            p.join()
+
+                    interactions_all = interactions_all[:]
         else:
             LOGGER.info('Include trajectory or use multi-model PDB file.')
 
+    if return_selection:
+        if indices is not None:
+            atoms_copy = atoms_copy[indices]
+            kwargs['selstr'] = atoms_copy.getSelstr()
+
+        return interactions_all, atoms_copy
+    
     return interactions_all
 
 
-def getResidueName(atom):
-    return f'{atom.getResname()}{atom.getResnum()}{atom.getChid()}'
+def getResidueName(atom, use_segname=False):
+    result = f'{atom.getResname()}{atom.getResnum()}{atom.getChid()}'
+    if use_segname:
+        result += f'{atom.getSegname()}'
+
+    return result
 
 
 class DictionaryList:
@@ -535,20 +747,27 @@ class DictionaryList:
     def keys(self):
         return self.values.keys()
 
+    def removeDuplicateKeys(self, criterion):
+        keysCopy = list(self.values.keys())
+        for key in keysCopy:
+            if criterion(self.values.keys(), key):
+                del self.values[key]
 
-def getResInfo(atoms):
+
+def getResInfo(atoms, **kwargs):
+    considered_atoms_sel = kwargs.pop('considered_atoms_sel', "protein")
     dict = {}
-    nums = atoms.select('protein').getResnums()
-    names = atoms.select('protein').getResnames()
-    chids = atoms.select('protein').getChids()
+    nums = atoms.select(considered_atoms_sel).getResnums()
+    names = atoms.select(considered_atoms_sel).getResnames()
+    chids = atoms.select(considered_atoms_sel).getChids()
 
     for i, num in enumerate(nums):
-        dict[num] = f"{names[i]}{num}{chids[i]}"
+        dict[num] = "{0}{1}{2}".format(names[i], num, chids[i])
 
     return dict
 
 
-def getWaterBridgeStatInfo(stats, atoms):
+def getWaterBridgeStatInfo(stats, atoms, **kwargs):
     """Converts calcWaterBridgesStatistic indices output to info output from stat.
 
     :arg stats: statistics returned by calcWaterBridgesStatistics, output='indices'
@@ -557,7 +776,7 @@ def getWaterBridgeStatInfo(stats, atoms):
     :arg atoms: Atomic object from which atoms are considered
     :type atoms: :class:`.Atomic`
     """
-    residueInfo = getResInfo(atoms)
+    residueInfo = getResInfo(atoms, **kwargs)
     infoOutput = {}
     for key, value in stats.items():
         x_id, y_id = key
@@ -576,18 +795,26 @@ def calcWaterBridgesStatistics(frames, trajectory, **kwargs):
     :type frames: list
 
     :arg output: return dictorinary whose keys are tuples of resnames or resids
-        default is 'resid'
+        default is 'indices'
     :type output: 'info' | 'indices'
 
     :arg filename: name of file to save statistic information if wanted
         default is None
     :type filename: string
+
+    :arg considered_atoms_sel: selection string for which atoms to consider
+        Default is **"protein"**
+    :type considered_atoms_sel: str
     """
     output = kwargs.pop('output', 'indices')
     filename = kwargs.pop('filename', None)
     if output not in ['info', 'indices']:
         raise TypeError('Output should be info or indices!')
 
+    considered_atoms_sel = kwargs.pop('considered_atoms_sel', None)
+    if considered_atoms_sel is not None:
+        trajectory.select(considered_atoms_sel)
+
     allCoordinates = trajectory.getCoordsets()
     interactionCount = DictionaryList(0)
     distances = DictionaryList([])
@@ -621,7 +848,11 @@ def calcWaterBridgesStatistics(frames, trajectory, **kwargs):
                 resNames[res_1] = res_1_name
                 resNames[res_2] = res_2_name
 
-    tableHeader = f'{"RES1":<15}{"RES2":<15}{"PERC":<10}{"DIST_AVG":<10}{"DIST_STD":<10}'
+    interactionCount.removeDuplicateKeys(
+        lambda keys, key: (key[1], key[0]) in keys)
+
+    tableHeader = '{0:<15}{1:<15}{2:<10}{3:<10}{4:<10}'.format(
+        "RES1", "RES2", "PERC", "DIST_AVG", "DIST_STD")
     LOGGER.info(tableHeader)
     info = {}
     file = open(filename, 'w') if filename else None
@@ -637,12 +868,15 @@ def calcWaterBridgesStatistics(frames, trajectory, **kwargs):
 
         outputKey = key
         x, y = key
-        if output == 'resname':
+        if output == 'info':
             outputKey = (resNames[x], resNames[y])
+            info[outputKey] = pairInfo
+        elif output == 'indices':
+            key1, key2 = (x, y), (y, x)
+            info[key1], info[key2] = pairInfo, pairInfo
 
-        info[outputKey] = pairInfo
-
-        tableRow = f'{resNames[x]:<15}{resNames[y]:<15}{percentage:<10.3f}{distAvg:<10.3f}{distStd:<10.3f}'
+        tableRow = '{0:<15}{1:<15}{2:<10.3f}{3:<10.3f}{4:<10.3f}'.format(
+            resNames[x], resNames[y], percentage, distAvg, distStd)
         LOGGER.info(tableRow)
         if file:
             file.write(tableRow + '\n')
@@ -688,7 +922,7 @@ def showWaterBridgeMatrix(data, metric):
         'distStd': 'Distance standard deviation'
     }
 
-    showAtomicMatrix(matrix)
+    showMatrix(matrix)
     plt.title(titles[metric])
 
 
@@ -708,7 +942,7 @@ def mofifyBeta(bridgeFrames, atoms):
     atoms.setBetas(0)
     for resnum, value in residueOccurances.items():
         residueAtoms = atoms.select(
-            f'resnum {resnum}')
+            'resnum {0}'.format(resnum))
         beta = value/len(bridgeFrames)
 
         residueAtoms.setBetas(beta)
@@ -723,8 +957,14 @@ def calcBridgingResiduesHistogram(frames, **kwargs):
     :arg clip: maximal number of residues on graph; to represent all set None
         default is 20
     :type clip: int
+
+    :arg use_segname: whether to use segname to label residues
+        default is False, because then the labels get long
+    :type use_segname: bool
     """
-    import matplotlib.pyplot as plt
+
+    show_plot = kwargs.pop('show_plot', False)
+    use_segname = kwargs.get('use_segname', False)
 
     clip = kwargs.pop('clip', 20)
     if clip == None:
@@ -733,7 +973,8 @@ def calcBridgingResiduesHistogram(frames, **kwargs):
     residuesWithCount = {}
     for frame in frames:
         frameResidues = set(reduceTo1D(
-            frame, getResidueName, lambda wb: wb.proteins))
+            frame, lambda x: getResidueName(x, use_segname=use_segname),
+            lambda wb: wb.proteins))
 
         for res in frameResidues:
             residuesWithCount[res] = residuesWithCount.get(res, 0) + 1
@@ -743,19 +984,26 @@ def calcBridgingResiduesHistogram(frames, **kwargs):
 
     labels, values = zip(*sortedResidues[-clip:])
 
-    plt.figure(figsize=(5, 3 + 0.11 * len(labels)))
-    plt.barh(labels, values)
-    plt.xlabel('Number of frame appearances')
-    plt.ylabel('Residue')
-    plt.title('Water bridging residues')
-    plt.tight_layout()
-    plt.margins(y=0.01)
-    plt.gca().xaxis.set_label_position('top')
-    plt.gca().xaxis.tick_top()
-    plt.show()
+    if show_plot:
+        import matplotlib.pyplot as plt
+        plt.figure(figsize=(5, 3 + 0.11 * len(labels)))
+        plt.barh(labels, values)
+        plt.xlabel('Number of frame appearances')
+        plt.ylabel('Residue')
+        plt.title('Water bridging residues')
+        plt.tight_layout()
+        plt.margins(y=0.01)
+        plt.gca().xaxis.set_label_position('top')
+        plt.gca().xaxis.tick_top()
+        if SETTINGS['auto_show']:
+            showFigure()
 
     return sortedResidues
 
+def showBridgingResiduesHistogram(frames, **kwargs):
+    kwargs['show_plot'] = True
+    return calcBridgingResiduesHistogram(frames, **kwargs)
+
 
 def getBridgingResidues(frames, residue):
     residuesWithCount = {}
@@ -816,7 +1064,7 @@ def getDistanceDistribution(frames, res_a, res_b, trajectory):
     return distances
 
 
-def getResidueLocationDistrubtion(frames, res_a, res_b):
+def getResidueLocationDistribution(frames, res_a, res_b):
     locationInfo = {"backbone": 0, "side": 0}
     result = {res_a: locationInfo.copy(), res_b: locationInfo.copy()}
 
@@ -862,8 +1110,7 @@ def calcWaterBridgesDistribution(frames, res_a, res_b=None, **kwargs):
         default is 'dict'
     :type output: 'dict' | 'indices'
     """
-    import matplotlib.pyplot as plt
-    
+    show_plot = kwargs.pop('show_plot', False)
     metric = kwargs.pop('metric', 'residues')
     trajectory = kwargs.pop('trajectory', None)
 
@@ -875,19 +1122,25 @@ def calcWaterBridgesDistribution(frames, res_a, res_b=None, **kwargs):
         'residues': lambda: getBridgingResidues(frames, res_a),
         'waters': lambda: getWaterCountDistribution(frames, res_a, res_b),
         'distance': lambda: getDistanceDistribution(frames, res_a, res_b, trajectory),
-        'location': lambda: getResidueLocationDistrubtion(frames, res_a, res_b)
+        'location': lambda: getResidueLocationDistribution(frames, res_a, res_b)
     }
 
     result = methods[metric]()
 
-    if metric in ['waters', 'distance']:
+    if metric in ['waters', 'distance'] and show_plot:
+        import matplotlib.pyplot as plt
         plt.hist(result, rwidth=0.95, density=True)
         plt.xlabel('Value')
         plt.ylabel('Probability')
-        plt.title(f'Distribution: {metric}')
-        plt.show()
+        plt.title('Distribution: {0}'.format(metric))
+        if SETTINGS['auto_show']:
+            showFigure()
 
-    return methods[metric]()
+    return result
+
+def showWaterBridgesDistribution(frames, res_a, res_b=None, **kwargs):
+    kwargs['show_plot'] = True
+    return calcWaterBridgesDistribution(frames, res_a, res_b, **kwargs)
 
 
 def savePDBWaterBridges(bridges, atoms, filename):
@@ -912,13 +1165,14 @@ def savePDBWaterBridges(bridges, atoms, filename):
     waterOxygens = reduceTo1D(
         bridges, lambda w: w.getIndex(), lambda b: b.waters)
     waterAtoms = atoms.select(
-        f'same residue as water within 1.6 of index {" ".join(map(str, waterOxygens))}')
+        'same residue as water within 1.6 of index {0}'.format(
+            " ".join(map(str, waterOxygens))))
 
     atomsToSave = proteinAtoms.toAtomGroup() + waterAtoms.toAtomGroup()
     return writePDB(filename, atomsToSave)
 
 
-def savePDBWaterBridgesTrajectory(bridgeFrames, atoms, filename, trajectory=None):
+def savePDBWaterBridgesTrajectory(bridgeFrames, atoms, filename, trajectory=None, max_proc=1):
     """Saves one PDB per frame with occupancy and beta on protein atoms and waters forming bridges in frame.
 
     :arg bridgeFrames: atomic output from calcWaterBridgesTrajectory
@@ -930,7 +1184,8 @@ def savePDBWaterBridgesTrajectory(bridgeFrames, atoms, filename, trajectory=None
     :arg filename: name of file to be saved; must end in .pdb
     :type filename: string
 
-    :arg trajectory: DCD trajectory (not needed for multimodal PDB)
+    :arg trajectory: trajectory data (not needed for multi-model PDB)
+    :type trajectory: :class:`.Trajectory', :class:`.Ensemble`, :class:`.Atomic`
     """
     if not trajectory and atoms.numCoordsets() < len(bridgeFrames):
         raise TypeError('Provide parsed trajectory!')
@@ -940,7 +1195,8 @@ def savePDBWaterBridgesTrajectory(bridgeFrames, atoms, filename, trajectory=None
     atoms = atoms.copy()
     mofifyBeta(bridgeFrames, atoms)
 
-    for frameIndex, frame in enumerate(bridgeFrames):
+    def saveBridgesFrame(trajectory, atoms, frameIndex, frame):
+        LOGGER.info('Frame: {0}'.format(frameIndex))
         if trajectory:
             coords = trajectory[frameIndex].getCoords()
             atoms.setCoords(coords)
@@ -949,23 +1205,45 @@ def savePDBWaterBridgesTrajectory(bridgeFrames, atoms, filename, trajectory=None
 
         waterAtoms = reduceTo1D(frame, sublistSel=lambda b: b.waters)
         waterResidues = atoms.select(
-            f'same residue as water within 1.6 of index {" ".join(map(lambda a: str(a.getIndex()), waterAtoms))}')
+            'same residue as water within 1.6 of index {0}'.format(
+                " ".join(map(lambda a: str(a.getIndex()), waterAtoms))))
 
         bridgeProteinAtoms = reduceTo1D(
             frame, lambda p: p.getResnum(), lambda b: b.proteins)
         atoms.setOccupancies(0)
-        atoms.select(
-            f'resid {" ".join(map(str, bridgeProteinAtoms))}').setOccupancies(1)
+        atoms.select('resid {0}'.format(
+            " ".join(map(str, bridgeProteinAtoms)))).setOccupancies(1)
 
         atomsToSave = atoms.select(
             'protein').toAtomGroup() + waterResidues.toAtomGroup()
 
         if trajectory:
-            writePDB(f'{filename}_{frameIndex}.pdb', atomsToSave)
+            writePDB('{0}_{1}.pdb'.format(filename, frameIndex),
+                     atomsToSave)
         else:
-            writePDB(f'{filename}_{frameIndex}.pdb',
+            writePDB('{0}_{1}.pdb'.format(filename, frameIndex),
                      atomsToSave, csets=frameIndex)
 
+    if max_proc == 1:
+        for frameIndex, frame in enumerate(bridgeFrames):
+            saveBridgesFrame(trajectory, atoms, frameIndex, frame)
+    else:
+        frameIndex = 0
+        numFrames = len(bridgeFrames)
+        while frameIndex < numFrames:
+            processes = []
+            for _ in range(max_proc):
+                p = mp.Process(target=saveBridgesFrame, args=(trajectory, atoms, frameIndex,
+                                                              bridgeFrames[frameIndex]))
+                p.start()
+                processes.append(p)
+
+                frameIndex += 1
+                if frameIndex >= numFrames:
+                    break
+
+            for p in processes:
+                p.join()
 
 def getBridgeIndicesString(bridge):
     return ' '.join(map(lambda a: str(a.getIndex()), bridge.proteins))\
@@ -992,7 +1270,7 @@ def saveWaterBridges(atomicBridges, filename):
     if isInfoOutput:
         info = getWaterBridgesInfoOutput(atomicBridges)
         for frameIndex, frame in enumerate(info):
-            file.write(f'FRAME {frameIndex}\n')
+            file.write('FRAME {0}\n'.format(frameIndex))
             for bridge in frame:
                 file.write(' '.join(map(str, bridge)) + '\n')
 
@@ -1046,3 +1324,242 @@ def parseWaterBridges(filename, atoms):
         bridgesFrames = bridgesFrames[0]
 
     return bridgesFrames
+
+
+def findClusterCenters(file_pattern, **kwargs):
+    """ Find molecules that are forming cluster in 3D space.
+    
+    :arg file_pattern: file pattern for anlaysis
+        it can include '*'
+        example:'file_*.pdb' will analyze file_1.pdb, file_2.pdb, etc.
+    :type file_pattern: str
+    
+    :arg selection: selection string
+        by default 'water and name "O.*"' is used
+    :type selection: str
+    
+    :arg distC: distance to other molecules
+    :type distC: int, float
+        default is 0.3
+        
+    :arg numC: min number of molecules in a cluster
+        default is 3
+    :type numC: int
+
+    :arg filename: filename for output pdb file with clusters
+        Default of **None** leads to
+        'clusters_'+file_pattern.split("*")[0]+'.pdb'
+    :type filename: str
+    """
+    
+    import glob
+    import numpy as np
+
+    selection = kwargs.pop('selection', 'water and name "O.*"')
+    distC = kwargs.pop('distC', 0.3)
+    numC = kwargs.pop('numC', 3)
+    filename = kwargs.pop('filename', None)
+    
+    matching_files = glob.glob(file_pattern)
+    matching_files.sort()
+    coords_all = parsePDB(matching_files[0]).select(selection).toAtomGroup()
+
+    for i in matching_files[1:]:
+        coords = parsePDB(i).select(selection).toAtomGroup()
+        coords_all += coords
+
+    removeResid = []
+    removeCoords = []
+    for ii in range(len(coords_all)):
+        if 'water' in selection.split() or np.any([water in selection.split() for water in DEFAULTS['water']]):
+            sel = coords_all.select('water within '+str(distC)+' of center', 
+                                    center=coords_all.getCoords()[ii])
+        else:
+            sel = coords_all.select(str(selection)+' within '+str(distC)+' of center',
+                                    center=coords_all.getCoords()[ii])
+        
+        if sel is not None and len(sel) <= int(numC):
+            removeResid.append(coords_all.getResnums()[ii])
+            removeCoords.append(list(coords_all.getCoords()[ii]))
+
+    if len(removeCoords) == coords_all.numAtoms():
+        raise ValueError('No waters were selected. You may need to align your trajectory \
+        or change default parameters for detecting water clusters (increase distC \
+        or decrease numC)')
+
+    selectedWaters = AtomGroup()
+    sel_waters = [] 
+
+    for j in coords_all.getCoordsets()[0].tolist():
+        if j not in removeCoords:
+            sel_waters.append(j)
+
+    coords_wat = np.array([sel_waters], dtype=float)
+    selectedWaters.setCoords(coords_wat)
+    selectedWaters.setNames(['DUM']*len(selectedWaters))
+    selectedWaters.setResnums(range(1, len(selectedWaters)+1))
+    selectedWaters.setResnames(['DUM']*len(selectedWaters))
+
+    if filename is None:
+        try:
+            filename = 'clusters_'+file_pattern.split("*")[0]+'.pdb'
+        except:
+            filename = 'clusters.pdb'
+
+    writePDB(filename, selectedWaters)
+    LOGGER.info("Results are saved in {0}.".format(filename))
+
+def filterStructuresWithoutWater(structures, min_water=0, filenames=None):
+    """This function will filter out structures from *structures* that have no water 
+    or fewer water molecules than *min_water*.
+    
+    :arg structures: list of :class:`.Atomic` structures to be filtered
+    :type structures: list
+
+    :arg min_water: minimum number of water O atoms, 
+        default is 0
+    :type min_water: int
+
+    :arg filenames: an optional list of filenames to filter too
+        This is an output argument
+    :type filenames: list
+    """
+
+    if not isinstance(structures, list):
+        raise TypeError('structures should be a list')
+    
+    if not np.all([isinstance(struct, Atomic) for struct in structures]):
+        raise ValueError('elements of structures should be Atomic objects')
+    
+    if not isinstance(min_water, int):
+        raise TypeError('min_water should be an integer')
+    
+    if filenames is None: filenames = []
+
+    if not isinstance(filenames, list):
+        raise TypeError('filenames should be None or a list')
+    
+    if len(filenames) not in [0, len(structures)]:
+        raise TypeError('filenames should have the same length as structures')
+    
+    if not np.all([isinstance(filename, str) for filename in filenames]):
+        raise ValueError('elements of filenames should be strings')
+    
+    if not np.all([os.path.exists(filename) for filename in filenames]):
+        raise ValueError('at least one of the filenames does not exist')
+    
+    have_filenames = len(filenames)>0
+
+    new_structures = []
+    numStructures = len(structures)
+    for i, struct in enumerate(reversed(structures)):
+        title = struct.getTitle()
+        waters = struct.select('water and name O')
+    
+        if waters == None:
+            LOGGER.warn(title+" doesn't contain water molecules")
+            if have_filenames:
+                filenames.pop(numStructures-i-1)
+            continue
+    
+        numWaters = waters.numAtoms()
+        if numWaters < min_water:
+            LOGGER.warn(title+" doesn't contain enough water molecules ({0})".format(numWaters))
+            if have_filenames:
+                filenames.pop(numStructures-i-1)
+            continue
+
+        new_structures.append(struct)
+
+    return list(reversed(new_structures))
+
+
+def selectSurroundingsBox(atoms, select, **kwargs):
+    """Select the surroundings of *select* within *atoms* using
+    a bounding box with optional *padding*.
+
+    :arg return_selstr: whether to return the final selstr
+        Default False
+    :type return_selstr: bool
+    """
+
+    if not isinstance(atoms, Atomic):
+        raise TypeError('atoms should be an Atomic object')
+
+    if isinstance(select, str):
+        select = atoms.select(select)
+
+    if not isinstance(select, Atomic):
+        raise TypeError('select should be a valid selection or selection string')
+
+    padding = kwargs.get('padding', 0)
+    if not isinstance(padding, Number):
+        raise TypeError('padding should be a number')
+    if padding < 0:
+        raise ValueError('padding should be a positive number')
+
+    return_selstr = kwargs.get('return_selstr', False)
+    if not isinstance(return_selstr, bool):
+        raise TypeError('return_selstr should be a bool')
+
+    minCoords = select.getCoords().min(axis=0)
+    maxCoords = select.getCoords().max(axis=0)
+
+    if padding > 0:
+        minCoords -= padding
+        maxCoords += padding
+
+    selstr = 'same residue as ((x `{0} to {1}`) and (y `{2} to {3}`) and (z `{4} to {5}`))'.format(
+        minCoords[0], maxCoords[0], minCoords[1], maxCoords[1], minCoords[2], maxCoords[2])
+
+    if return_selstr:
+        return selstr
+    return atoms.select(selstr)
+
+
+def findCommonSelectionTraj(atoms, traj, selstr, **kwargs):
+    """Select *selstr* within *atoms* for each frame in *traj*
+    using a bounding box with optional *padding*.
+
+    :arg expand_selection: whether to expand selections with
+        :meth:`.selectSurroundingsBox`. Default False
+    :type expand_selection: bool
+
+    Returns the common selection and corresponding indices and
+    optionally the corresponding selstr if *return_selstr* is **True**
+    """
+
+    if not isinstance(atoms, Atomic):
+        raise TypeError('atoms should be an Atomic object')
+
+    if not isinstance(selstr, str):
+        raise TypeError('selstr should be a string')
+
+    expand_selection = kwargs.get('expand_selection', False)
+    if not isinstance(expand_selection, bool):
+        raise TypeError('expand_selection should be a bool')
+
+    return_selstr = kwargs.pop('return_selstr', False) # not passed on
+    if not isinstance(return_selstr, bool):
+        raise TypeError('return_selstr should be a bool')
+
+    indices = None
+    if selstr is not None:
+        LOGGER.info('Finding common selection')
+        indices = []
+        for frame0 in traj:
+            atoms_copy = atoms.copy()
+            atoms_copy.setCoords(frame0.getCoords())
+            selection = atoms_copy.select(selstr)
+
+            if expand_selection:
+                selection = selectSurroundingsBox(atoms_copy, selection, **kwargs)
+
+            indices.extend(list(selection.getIndices()))
+
+        indices = np.unique(indices)
+        selection = atoms_copy[indices]
+
+        if return_selstr:
+            return selection, indices, selstr
+        return selection, indices
