@@ -3,13 +3,10 @@
 
 from numbers import Integral
 
-from numpy import all, zeros, dtype, array, char, cumsum, ceil, reshape
-from numpy import where, sort, concatenate, vstack, isscalar, chararray
-
-from Bio import AlignIO
+from numpy import all, zeros, dtype, array, char, cumsum, reshape
+from numpy import vstack, isscalar
 
 from prody import LOGGER, PY3K
-from prody.atomic import Atomic
 from prody.utilities import toChararray, pystr, splitSeqLabel
 from .sequence import Sequence
 
@@ -156,7 +153,7 @@ class MSA(object):
         if isinstance(rows, list):
             rows = self.getIndex(rows) or rows
         elif isinstance(rows, int):
-            return Sequence(self._msa[rows, cols].tostring(),
+            return Sequence(self._msa[rows, cols].tobytes(),
                             self._labels[rows])
         elif isinstance(rows, str):
             try:
@@ -166,7 +163,7 @@ class MSA(object):
                                .format(index))
             else:
                 if isinstance(rows, int):
-                    return Sequence(self._msa[rows, cols].tostring(),
+                    return Sequence(self._msa[rows, cols].tobytes(),
                                     self._labels[rows])
 
         if cols is None:
@@ -548,7 +545,7 @@ def refineMSA(msa, index=None, label=None, rowocc=None, seqid=None, colocc=None,
                 from prody.utilities import GAP_PENALTY, GAP_EXT_PENALTY, ALIGNMENT_METHOD
 
                 chseq = chain.getSequence()
-                algn = alignBioPairwise(pystr(arr[index].tostring().upper()), pystr(chseq),
+                algn = alignBioPairwise(pystr(arr[index].tobytes().upper()), pystr(chseq),
                                         "local",
                                         MATCH_SCORE, MISMATCH_SCORE,
                                         GAP_PENALTY, GAP_EXT_PENALTY,
