@@ -3,24 +3,23 @@
 import os
 
 import numpy as np
-from numpy.testing import *
-try:
-    import numpy.testing.decorators as dec
-except ImportError:
-    from numpy.testing import dec
+from numpy.testing import assert_equal, assert_allclose
 
-from prody import *
+from prody.utilities import importDec
+dec = importDec()
+
+import prody
 from prody import LOGGER
-from prody.utilities import which
+from prody.proteins import parsePDB, writePDB, PDBParseError
 from prody.tests import TEMPDIR, unittest
-from prody.tests.datafiles import *
+from prody.tests.datafiles import parseDatafile, pathDatafile, DATA_FILES
 
 LOGGER.verbosity = 'none'
 
 class TestParsePDB(unittest.TestCase):
 
     def setUp(self):
-        """Set PDB file data and parse the PDB file."""
+        """Set PDB file data."""
 
         self.pdb = DATA_FILES['multi_model_truncated']
         self.one = DATA_FILES['oneatom']
@@ -30,6 +29,11 @@ class TestParsePDB(unittest.TestCase):
         self.hex = DATA_FILES['hex']
         self.h36 = DATA_FILES['h36']
 
+        self.probes = DATA_FILES['probes']
+
+        self.altlocs = DATA_FILES['6flr']
+        self.his_selstr = 'resname HIS and chain B and resnum 234 and name CA'
+
     def testUsualCase(self):
         """Test the outcome of a simple parsing scenario."""
 
@@ -61,7 +65,7 @@ class TestParsePDB(unittest.TestCase):
         path = pathDatafile(self.pdb['file'])
         self.assertRaises(TypeError, parsePDB, path, model='0')
         self.assertRaises(ValueError, parsePDB, path, model=-1)
-        self.assertRaises(proteins.PDBParseError, parsePDB, path,
+        self.assertRaises(PDBParseError, parsePDB, path,
                           model=self.pdb['models']+1)
         self.assertIsNone(parsePDB(path, model=0),
             'parsePDB failed to parse no coordinate sets')
@@ -106,17 +110,25 @@ class TestParsePDB(unittest.TestCase):
                         'failed to parse correct number of "bb" atoms')
 
     def testAgArgument(self):
-        """Test outcome of valid and invalid *ag* arguments."""
+        """Test outcome of 2 invalid and 2 valid *ag* arguments."""
 
         path = pathDatafile(self.pdb['file'])
         self.assertRaises(TypeError, parsePDB, path, ag='AtomGroup')
+
         ag = prody.AtomGroup('One atom')
         ag.setCoords(np.array([[0., 0., 0.]]))
         self.assertRaises(ValueError, parsePDB, path, ag=ag)
+
         ag = prody.AtomGroup('Test')
         self.assertEqual(parsePDB(path, ag=ag).numAtoms(),
             self.pdb['atoms'],
             'parsePDB failed to parse correct number of atoms')
+        
+        ag = prody.AtomGroup('Test')
+        ag.setCoords(np.array([[0., 0., 0.]]*self.pdb['atoms']))
+        self.assertEqual(parsePDB(path, ag=ag).numAtoms(),
+            self.pdb['atoms'],
+            'parsePDB failed to parse correct number of atoms')
 
     def testAgArgMultiModel(self):
         """Test number of coordinate sets when using *ag* arguments."""
@@ -186,18 +198,79 @@ class TestParsePDB(unittest.TestCase):
         serial = '100000'
         self.assertEqual(str(parsePDB(path).getSerials()[100000-1]),
              serial, 'parsePDB failed to parse Hybrid36 serial number')
-'''
-    def testBiomolArgument(self):
+        
+    def testAltlocAllToMultiAtoms(self):
+        """Test number of coordinate sets and atoms with altloc='all'."""
 
-        self.assertRaises(ValueError, parsePDB, self.one['path'],
-                          biomol=True)
+        path = pathDatafile(self.altlocs['file'])
 
+        ag = parsePDB(path, altloc="all")
+        self.assertEqual(ag.numAtoms(), self.altlocs['atoms_altloc'],
+            'parsePDB failed to parse correct number of atoms with altloc "all"')
+        self.assertEqual(ag.numCoordsets(), 1,
+            'parsePDB failed to parse correct number of coordsets (1) with altloc "all"')
 
-    def testSecondaryArgument(self):
+        hisB234 = ag.select(self.his_selstr)
+        self.assertEqual(hisB234.numAtoms(), self.altlocs['num_altlocs'],
+            'parsePDB failed to parse correct number of His B234 CA atoms (2) with altloc "all"')
+
+        self.assertEqual(hisB234.getAnisous().shape, (self.altlocs['num_altlocs'], 6),
+            'parsePDB failed to have right shape for His B234 CA atoms getAnisous (2, 6) with altloc "all"')
+
+        assert_allclose(hisB234.getAnisous()[0], self.altlocs['anisousA'][0],
+            err_msg='parsePDB failed to have right His B234 CA atoms getAnisous A with altloc "all"')
+
+        assert_allclose(hisB234.getAnisous()[1], self.altlocs['anisousB'][0],
+            err_msg='parsePDB failed to have right His B234 CA atoms getAnisous B with altloc "all"')
+        
+    def testAltlocNoneToMultiCoordsets(self):
+        """Test number of coordinate sets and atoms with altloc=None."""
+
+        path = pathDatafile(self.altlocs['file'])
+
+        ag = parsePDB(path, altloc=None)
+        self.assertEqual(ag.numAtoms(), self.altlocs['atoms_single'],
+            'parsePDB failed to parse correct number of atoms with altloc None')
+        self.assertEqual(ag.numCoordsets(), self.altlocs['num_altlocs'],
+            'parsePDB failed to parse correct number of coordsets (2) with altloc None')
+
+        hisB234 = ag.select(self.his_selstr)
+        self.assertEqual(hisB234.numAtoms(), 1,
+            'parsePDB failed to parse correct number of His B234 CA atoms (1) with altloc None')
+
+        self.assertEqual(hisB234.getAnisous().shape, (1, 6),
+            'parsePDB failed to have right shape for His B234 CA atoms getAnisous (1, 6) with altloc None')
+
+        assert_allclose(hisB234.getAnisous(), self.altlocs['anisousA'],
+            err_msg='parsePDB failed to have right His B234 CA atoms getAnisous A with altloc None')
+
+        hisB234.setACSIndex(1)
+
+        assert_allclose(hisB234.getAnisous(), self.altlocs['anisousB'],
+            err_msg='parsePDB failed to have right His B234 CA atoms getAnisous B with altloc None')
+
+    def testParseLongResnameFalse(self):
+        """Test the outcome of parsing with clipping long resnames."""
+
+        ag = parseDatafile(self.probes['file'])
+
+        self.assertIsInstance(ag, prody.AtomGroup,
+            'parsePDB failed to return an AtomGroup instance')
+
+        self.assertEqual(ag.getResnames()[0],
+                         self.probes['short_resname'])
+
+    def testParseLongResnameTrue(self):
+        """Test the outcome of parsing with clipping long resnames."""
+
+        ag = parseDatafile(self.probes['file'], long_resname=True)
+
+        self.assertIsInstance(ag, prody.AtomGroup,
+            'parsePDB failed to return an AtomGroup instance')
+
+        self.assertEqual(ag.getResnames()[0],
+                         self.probes['long_resname'])
 
-        self.assertRaises(ValueError, parsePDB, self.one['path'],
-                          secondary=True)
-'''
 
 class TestWritePDB(unittest.TestCase):
 
@@ -209,7 +282,13 @@ class TestWritePDB(unittest.TestCase):
         self.ag = parsePDB(self.pdb['path'])
         self.tmp = os.path.join(TEMPDIR, 'test.pdb')
 
+        self.ens = prody.PDBEnsemble()
+        self.ens.setAtoms(self.ag)
+        self.ens.setCoords(self.ag.getCoords())
+        self.ens.addCoordset(self.ag.getCoordsets())
+
         self.ubi = parsePDB(DATA_FILES['1ubi']['path'], secondary=True)
+        self.probes = parsePDB(DATA_FILES['probes']['path'], long_resname=True)
 
         self.hex = parsePDB(DATA_FILES['hex']['path'])
         self.h36 = parsePDB(DATA_FILES['h36']['path'])
@@ -217,6 +296,14 @@ class TestWritePDB(unittest.TestCase):
         self.hex_ter = parsePDB(DATA_FILES['hex_ter']['path'])
         self.h36_ter = parsePDB(DATA_FILES['h36_ter']['path'])
 
+        self.altlocs = DATA_FILES['6flr']
+        self.altloc_full = parsePDB(self.altlocs['path'], altloc=None,
+                                    secondary=False)
+        self.altloc_sel = DATA_FILES['6flr_sel']['path']
+
+        self.sort_sel = DATA_FILES['6zu5_sel']
+        self.sort_sel_ag = parsePDB(self.sort_sel['path'])
+
     msg = 'user does not have write access to temp dir {0:s}'.format(TEMPDIR)
 
     @dec.slow
@@ -233,7 +320,7 @@ class TestWritePDB(unittest.TestCase):
         self.assertEqual(self.ag.numAtoms(), out.numAtoms(),
             'writePDB failed to write correct number of atoms')
         self.assertEqual(self.ag.numCoordsets(), out.numCoordsets(),
-            'writePDB failed to write correct number of atoms')
+            'writePDB failed to write correct number of coordsets')
 
     @dec.slow
     @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
@@ -296,6 +383,15 @@ class TestWritePDB(unittest.TestCase):
         ubi_new = parsePDB(out, secondary=True)
         self.assertListEqual(list(self.ubi.getSecstrs()), list(ubi_new.getSecstrs()))
 
+    @dec.slow
+    @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
+    def testWritingLongResnums(self):
+        """Test if output from writing secstrs is as expected."""
+
+        out = writePDB(self.tmp, self.ubi)
+        ubi_new = parsePDB(out, secondary=True)
+        self.assertListEqual(list(self.ubi.getSecstrs()), list(ubi_new.getSecstrs()))
+
     @dec.slow
     @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
     def testModelArgument(self):
@@ -368,7 +464,81 @@ class TestWritePDB(unittest.TestCase):
 
         serial_A0000_line = lines[100000]
         self.assertEqual(serial_A0000_line[6:11], 'A0000',
-            'writePDB failed to write correct h36 serial') 
+            'writePDB failed to write correct h36 serial')
+        
+    def testWritingAltlocModels(self):
+        """Test if output from writing hexadecimal with TER lines is as expected."""
+
+        hisB234 = self.altloc_full.select('resname HIS and chain B and resnum 234 and name CA')
+        out = writePDB(self.tmp, hisB234)
+
+        fi = open(out, 'r')
+        lines1 = fi.readlines()
+        fi.close()
+
+        fi = open(self.altloc_sel, 'r')
+        lines2 = fi.readlines()
+        fi.close()
+        
+        self.assertEqual(lines1[3], lines2[3],
+            'writePDB failed to write correct ANISOU line 3 for 6flr selection with altloc None')
+        
+        self.assertEqual(lines1[7], lines2[7],
+            'writePDB failed to write correct ANISOU line 7 for 6flr selection with altloc None')
+        
+    def testWriteEnsembleToPDB(self):
+        """Test that writePDB can handle ensembles."""
+
+        out = writePDB(self.tmp, self.ens)
+        out = parsePDB(out)
+        self.assertEqual(out.numCoordsets(), self.ens.numCoordsets(),
+            'failed to write correct number of models from ensemble')
+        assert_equal(out.getCoords(), self.ag.getCoordsets(0),
+                'failed to write ensemble model 1 coordinates correctly')
+
+    @dec.slow
+    @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
+    def testWritingAtomMap(self):
+        """Test if output from writing a sorted AtomMap works and is as expected."""
+
+        sorted_sel = prody.sortAtoms(self.sort_sel_ag, 'chain')
+        out = writePDB(self.tmp, sorted_sel)
+        new = parsePDB(out)
+        self.assertListEqual(list(new.getChids()), self.sort_sel['sorted_order'])
+
+    @dec.slow
+    @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
+    def testWritingParsingLongResnames(self):
+        """Test if parsing output is the same as parsing original file."""
+
+        out = writePDB(self.tmp, self.probes, long_resname=True)
+        self.assertEqual(self.tmp, out,
+            'writePDB failed to return correct output filename')
+        self.assertTrue(os.path.isfile(out),
+            'writePDB failed to write output')
+
+        probes = parsePDB(out, long_resname=True)
+        self.assertEqual(self.probes.numAtoms(), probes.numAtoms(),
+            'writePDB failed to write correct number of atoms')
+        self.assertEqual(str(self.probes.getResnames()[0]), str(probes.getResnames()[0]),
+            'writePDB failed to write long resnames')
+
+    @dec.slow
+    @unittest.skipUnless(os.access(TEMPDIR, os.W_OK), msg)
+    def testWritingParsingLongResnamesFalse(self):
+        """Test if parsing output is the same as parsing original file."""
+
+        out = writePDB(self.tmp, self.probes)
+        self.assertEqual(self.tmp, out,
+            'writePDB failed to return correct output filename')
+        self.assertTrue(os.path.isfile(out),
+            'writePDB failed to write output')
+
+        probes = parsePDB(out, long_resname=True)
+        self.assertEqual(self.probes.numAtoms(), probes.numAtoms(),
+            'writePDB failed to write correct number of atoms')
+        self.assertEqual(probes.getResnames()[0], self.probes.getResnames()[0][:3],
+            'writePDB failed to write short resnames')
 
     @dec.slow
     def tearDown(self):
