@@ -10,7 +10,7 @@ from os.path import getsize
 import datetime
 
 import numpy as np
-from numpy import float32, fromstring
+from numpy import float32, frombuffer
 
 from prody.atomic import Atomic
 from prody.ensemble import Ensemble
@@ -270,8 +270,8 @@ class DCDFile(TrajFile):
 
         n_floats = self._n_floats
         n_atoms = self._n_atoms
-        xyz = fromstring(self._file.read(self._itemsize * n_floats),
-                            self._dtype)
+        xyz = frombuffer(self._file.read(self._itemsize * n_floats),
+                         self._dtype).copy()
         if len(xyz) != n_floats:
             return None
         xyz = xyz.reshape((3, n_atoms+2)).T[1:-1,:]
@@ -291,7 +291,7 @@ class DCDFile(TrajFile):
 
         if self._unitcell:
             self._file.read(4)
-            unitcell = fromstring(self._file.read(48), dtype=np.float64)
+            unitcell = frombuffer(self._file.read(48), dtype=np.float64)
             unitcell = unitcell[[0,2,5,1,3,4]]
             if np.all(abs(unitcell[3:]) <= 1):
                 # This file was generated by CHARMM, or by NAMD > 2.5, with the angle */
@@ -317,7 +317,7 @@ class DCDFile(TrajFile):
             n_atoms = self._n_atoms
             n_csets = self._n_csets
             data = self._file.read(self._itemsize * n_floats * n_csets)
-            data = fromstring(data, self._dtype)
+            data = frombuffer(data, self._dtype).copy()
             if len(data) > n_floats * n_csets:
                 n_csets = len(data)/n_floats
                 data = data[:n_csets]
