@@ -1,10 +1,10 @@
 """This module defines miscellaneous utility functions."""
 import re
 
-from numpy import unique, linalg, diag, sqrt, dot, chararray, divide, zeros_like, zeros, allclose, ceil, abs
-from numpy import diff, where, insert, nan, isnan, loadtxt, array, round, average, min, max, delete, vstack
+from numpy import unique, linalg, diag, sqrt, dot, zeros, allclose, abs, bytes_
+from numpy import diff, where, insert, nan, isnan, loadtxt, array, round, average, min, max
 from numpy import sign, arange, asarray, ndarray, subtract, power, sum, isscalar, empty, triu, tril, median
-from numpy import alltrue
+from numpy import all
 from collections import Counter
 import numbers
 
@@ -21,7 +21,8 @@ __all__ = ['Everything', 'Cursor', 'ImageCursor', 'rangeString', 'alnum', 'impor
            'getDataPath', 'openData', 'chr2', 'toChararray', 'interpY', 'cmp', 'pystr',
            'getValue', 'indentElement', 'isPDB', 'isURL', 'isListLike', 'isSymmetric', 'makeSymmetric',
            'getDistance', 'fastin', 'createStringIO', 'div0', 'wmean', 'bin2dec', 'wrapModes', 
-           'fixArraySize', 'decToHybrid36', 'hybrid36ToDec', 'DTYPE', 'checkIdentifiers', 'split', 'mad']
+           'fixArraySize', 'decToHybrid36', 'hybrid36ToDec', 'DTYPE', 'checkIdentifiers', 'split', 'mad',
+           'importDec', 'impLoadModule']
 
 DTYPE = array(['a']).dtype.char  # 'S' for PY2K and 'U' for PY3K
 CURSORS = []
@@ -421,8 +422,12 @@ def pystr(a):
     return b
 
 def getDataPath(filename):
+    if PY3K:
+        import importlib
+        path = importlib.util.find_spec('prody.utilities.datafiles').submodule_search_locations[0]
+        return '%s/%s' %(path, filename)
     import pkg_resources
-    return pkg_resources.resource_filename('prody.utilities', 'datafiles/%s'%filename)
+    return pkg_resources.resource_filename('prody.utilities', 'datafiles/%s'%filename)      
 
 def openData(filename, mode='r'):
     return open(getDataPath(filename), mode)
@@ -453,7 +458,7 @@ def toChararray(arr, aligned=False):
     if ndim != 2:
         n_seq = shape[0]
         l_seq = dtype_.itemsize
-        new_arr = chararray((n_seq, l_seq))
+        new_arr = empty((n_seq, l_seq), dtype=bytes_)
         for i, s in enumerate(arr):
             for j in range(l_seq):
                 if j < len(s):
@@ -654,7 +659,9 @@ def checkIdentifiers(*pdb, **kwargs):
             LOGGER.warn('{0} is not a valid identifier.'.format(repr(pid)))
             append(None)
         else:
-            if format != 'emd' and not (len(pid) == 4 and pid.isalnum()):
+            if format != 'emd' and not ((len(pid) == 4 and pid.isalnum()) 
+                                        or (len(pid) == 12 and pid.startswith('pdb_') 
+                                            and pid[3] == '_')):
                 LOGGER.warn('{0} is not a valid identifier.'
                             .format(repr(pid)))
                 append(None)
@@ -726,7 +733,7 @@ def hybrid36ToDec(x, resnum=False):
     if not isinstance(x, str):
         raise TypeError('x should be a string')
     
-    isnumeric = alltrue([y.isdigit() for y in x])
+    isnumeric = all([y.isdigit() for y in x])
     if isnumeric:
         return int(x)
 
@@ -784,3 +791,40 @@ def mad(x):
                 return median(abs(x - med))
     
     return _mad(x)
+
+
+def importDec():
+    """Returns an imported module equivalent to numpy testing decorators."""
+
+    try:
+        import numpy.testing.decorators as dec
+    except ModuleNotFoundError:
+        try:
+            from numpy.testing import dec
+        except ImportError:
+            try:
+                import numpy.testing._private.decorators as dec
+            except ModuleNotFoundError:
+                from pytest import mark as dec    
+
+    return dec
+
+
+def impLoadModule(name, cmd, path):
+    """Returns an  an imported module equivalent to imp."""
+
+    if not name.endswith('.'):
+        name += '.'
+
+    try:
+        import imp
+        mod = imp.load_module(name + cmd,
+                              *imp.find_module(cmd, [path]))
+    except ImportError:
+        import importlib
+        loader = importlib.machinery.SourceFileLoader(name + cmd,
+                                                      path + cmd + '.py')
+        mod = importlib.util.types.ModuleType(loader.name)
+        loader.exec_module(mod)
+
+    return mod
